@@ -45,22 +45,24 @@
 // We also put the type of the variable in the namespace, so that
 // people can't DECLARE_int32 something that they DEFINE_bool'd
 // elsewhere.
-#ifndef BASE_COMMANDLINEFLAGS_H__
-#define BASE_COMMANDLINEFLAGS_H__
+#ifndef BASE_COMMANDLINEFLAGS_H_
+#define BASE_COMMANDLINEFLAGS_H_
 
+#include "config.h"
 #include <string>
 #include <string.h>               // for memchr
+#include <stdlib.h>               // for getenv
 #include "base/basictypes.h"
 
 #define DECLARE_VARIABLE(type, name)                                          \
   namespace FLAG__namespace_do_not_use_directly_use_DECLARE_##type##_instead {  \
-  extern type FLAGS_##name;                                                   \
+  extern PERFTOOLS_DLL_DECL type FLAGS_##name;                                \
   }                                                                           \
   using FLAG__namespace_do_not_use_directly_use_DECLARE_##type##_instead::FLAGS_##name
 
 #define DEFINE_VARIABLE(type, name, value, meaning) \
   namespace FLAG__namespace_do_not_use_directly_use_DECLARE_##type##_instead {  \
-  type FLAGS_##name(value);                                                   \
+  PERFTOOLS_DLL_DECL type FLAGS_##name(value);                                \
   char FLAGS_no##name;                                                        \
   }                                                                           \
   using FLAG__namespace_do_not_use_directly_use_DECLARE_##type##_instead::FLAGS_##name
@@ -123,4 +125,4 @@
 #define EnvToInt64(envname, dflt)  \
   (!getenv(envname) ? (dflt) : strtoll(getenv(envname), NULL, 10))
 
-#endif  // BASE_COMMANDLINEFLAGS_H__
+#endif  // BASE_COMMANDLINEFLAGS_H_
