@@ -37,14 +37,26 @@
 // it should not be used when performance is key.
 
 #include "base/low_level_alloc.h"
+#include "base/dynamic_annotations.h"
 #include "base/spinlock.h"
 #include "base/logging.h"
+#include "malloc_hook-inl.h"
 #include <google/malloc_hook.h>
-#include <unistd.h>
 #include <errno.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#ifdef HAVE_MMAP
 #include <sys/mman.h>
+#endif
 #include <new>                   // for placement-new
 
+// On systems (like freebsd) that don't define MAP_ANONYMOUS, use the old
+// form of the name instead.
+#ifndef MAP_ANONYMOUS
+# define MAP_ANONYMOUS MAP_ANON
+#endif
+
 // A first-fit allocator with amortized logarithmic free() time.
 
 // ---------------------------------------------------------------------------
@@ -91,6 +103,7 @@ static int IntLog2(size_t size, size_t base) {
 // Return a random integer n:  p(n)=1/(2**n) if 1 <= n; p(n)=0 if n < 1.
 static int Random() {
   static int32 r = 1;         // no locking---it's not critical
+  ANNOTATE_BENIGN_RACE(&r, "benign race, not critical.");
   int result = 1;
   while ((((r = r*1103515245 + 12345) >> 30) & 1) == 0) {
     result++;
@@ -187,6 +200,11 @@ struct LowLevelAlloc::Arena {
 // pointer.
 static struct LowLevelAlloc::Arena default_arena;
 
+// A non-malloc-hooked arena: used only to allocate metadata for arenas that
+// do not want malloc hook reporting, so that for them there's no malloc hook
+// reporting even during arena creation.
+static struct LowLevelAlloc::Arena unhooked_arena;
+
 // magic numbers to identify allocated and unallocated blocks
 static const intptr_t kMagicAllocated = 0x4c833e95;
 static const intptr_t kMagicUnallocated = ~kMagicAllocated;
@@ -216,18 +234,28 @@ static void ArenaInit(LowLevelAlloc::Arena *arena) {
     arena->freelist.levels = 0;
     memset(arena->freelist.next, 0, sizeof (arena->freelist.next));
     arena->allocation_count = 0;
-    arena->flags = 0;
+    if (arena == &default_arena) {
+      // Default arena should be hooked, e.g. for heap-checker to trace
+      // pointer chains through objects in the default arena.
+      arena->flags = LowLevelAlloc::kCallMallocHook;
+    } else {
+      arena->flags = 0;   // other arenas' flags may be overridden by client,
+                          // but unhooked_arena will have 0 in 'flags'.
+    }
   }
 }
 
 // L < meta_data_arena->mu
 LowLevelAlloc::Arena *LowLevelAlloc::NewArena(int32 flags,
                                               Arena *meta_data_arena) {
-  if (meta_data_arena == 0) {
-    meta_data_arena = &default_arena;
+  RAW_CHECK(meta_data_arena != 0, "must pass a valid arena");
+  if (meta_data_arena == &default_arena  &&
+      (flags & LowLevelAlloc::kCallMallocHook) == 0) {
+    meta_data_arena = &unhooked_arena;
   }
   // Arena(0) uses the constructor for non-static contexts
-  Arena *result = new (Alloc(sizeof (*result), meta_data_arena)) Arena(0);
+  Arena *result =
+    new (AllocWithArena(sizeof (*result), meta_data_arena)) Arena(0);
   ArenaInit(result);
   result->flags = flags;
   return result;
@@ -235,7 +263,7 @@ LowLevelAlloc::Arena *LowLevelAlloc::NewArena(int32 flags,
 
 // L < arena->mu, L < arena->arena->mu
 bool LowLevelAlloc::DeleteArena(Arena *arena) {
-  RAW_CHECK(arena != 0 && arena != &default_arena,
+  RAW_CHECK(arena != 0 && arena != &default_arena && arena != &unhooked_arena,
             "may not delete default arena");
   arena->mu.Lock();
   bool empty = (arena->allocation_count == 0);
@@ -348,11 +376,8 @@ void LowLevelAlloc::Free(void *v) {
 
 // allocates and returns a block of size bytes, to be freed with Free()
 // L < arena->mu
-void *LowLevelAlloc::Alloc(size_t request, Arena *arena) {
+void *DoAllocWithArena(size_t request, LowLevelAlloc::Arena *arena) {
   void *result = 0;
-  if (arena == 0) {
-    arena = &default_arena;
-  }
   if (request != 0) {
     AllocList *s;       // will point to region that satisfies request
     arena->mu.Lock();
@@ -363,7 +388,7 @@ void *LowLevelAlloc::Alloc(size_t request, Arena *arena) {
       // find the minimum levels that a block of this size must have
       int i = LLA_SkiplistLevels(req_rnd, arena->min_size, false) - 1;
       if (i < arena->freelist.levels) {   // potential blocks exist
-        AllocList *before = &arena->freelist; // predecessor of s
+        AllocList *before = &arena->freelist;  // predecessor of s
         while ((s = Next(i, before, arena)) != 0 && s->header.size < req_rnd) {
           before = s;
         }
@@ -374,22 +399,24 @@ void *LowLevelAlloc::Alloc(size_t request, Arena *arena) {
       // we unlock before mmap() both because mmap() may call a callback hook,
       // and because it may be slow.
       arena->mu.Unlock();
-      size_t new_pages_size = RoundUp(req_rnd, arena->pagesize);
+      // mmap generous 64K chunks to decrease
+      // the chances/impact of fragmentation:
+      size_t new_pages_size = RoundUp(req_rnd, arena->pagesize * 16);
       void *new_pages = mmap(0, new_pages_size,
                      PROT_WRITE|PROT_READ, MAP_ANONYMOUS|MAP_PRIVATE, -1, 0);
       RAW_CHECK(new_pages != MAP_FAILED, "mmap error");
+      arena->mu.Lock();
       s = reinterpret_cast<AllocList *>(new_pages);
       s->header.size = new_pages_size;
       // Pretend the block is allocated; call AddToFreelist() to free it.
       s->header.magic = Magic(kMagicAllocated, &s->header);
       s->header.arena = arena;
-      arena->mu.Lock();
       AddToFreelist(&s->levels, arena);  // insert new region into free list
     }
     AllocList *prev[kMaxLevel];
     LLA_SkiplistDelete(&arena->freelist, s, prev);    // remove from free list
     // s points to the first free region that's big enough
-    if (req_rnd + arena->min_size <= s->header.size) { // big enough to split
+    if (req_rnd + arena->min_size <= s->header.size) {  // big enough to split
       AllocList *n = reinterpret_cast<AllocList *>
                         (req_rnd + reinterpret_cast<char *>(s));
       n->header.size = s->header.size - req_rnd;
@@ -404,8 +431,31 @@ void *LowLevelAlloc::Alloc(size_t request, Arena *arena) {
     arena->mu.Unlock();
     result = &s->levels;
   }
-  if ((arena->flags & kCallMallocHook) != 0) {
+  ANNOTATE_NEW_MEMORY(result, request);
+  return result;
+}
+
+void *LowLevelAlloc::Alloc(size_t request) {
+  void *result = DoAllocWithArena(request, &default_arena);
+  if ((default_arena.flags & kCallMallocHook) != 0) {
+    // this call must be directly in the user-called allocator function
+    // for MallocHook::GetCallerStackTrace to work properly
     MallocHook::InvokeNewHook(result, request);
   }
   return result;
 }
+
+void *LowLevelAlloc::AllocWithArena(size_t request, Arena *arena) {
+  RAW_CHECK(arena != 0, "must pass a valid arena");
+  void *result = DoAllocWithArena(request, arena);
+  if ((arena->flags & kCallMallocHook) != 0) {
+    // this call must be directly in the user-called allocator function
+    // for MallocHook::GetCallerStackTrace to work properly
+    MallocHook::InvokeNewHook(result, request);
+  }
+  return result;
+}
+
+LowLevelAlloc::Arena *LowLevelAlloc::DefaultArena() {
+  return &default_arena;
+}
