@@ -33,7 +33,7 @@
 //
 // Module for detecing heap (memory) leaks.
 //
-// For full(er) information, see doc/heap-checker.html
+// For full(er) information, see doc/heap_checker.html
 //
 // This module can be linked into programs with
 // no slowdown caused by this unless you activate the leak-checker:
@@ -48,31 +48,45 @@
 // a memory leak at program-exit, it will print instructions on how
 // to track down the leak.
 
-
-#ifndef BASE_HEAP_CHECKER_H__
-#define BASE_HEAP_CHECKER_H__
+#ifndef BASE_HEAP_CHECKER_H_
+#define BASE_HEAP_CHECKER_H_
 
 #include <sys/types.h>  // for size_t
 #include <stdint.h>     // for uintptr_t
+#include <stdarg.h>     // for va_list
 #include <vector>
 
+// Annoying stuff for windows -- makes sure clients can import these functions
+#ifndef PERFTOOLS_DLL_DECL
+# ifdef WIN32
+#   define PERFTOOLS_DLL_DECL  __declspec(dllimport)
+# else
+#   define PERFTOOLS_DLL_DECL
+# endif
+#endif
 
-class HeapLeakChecker {
- public:  // Static functions for working with (whole-program) leak checking.
+
+// The class is thread-safe with respect to all the provided static methods,
+// as well as HeapLeakChecker objects: they can be accessed by multiple threads.
+class PERFTOOLS_DLL_DECL HeapLeakChecker {
+ public:
+
+  // ----------------------------------------------------------------------- //
+  // Static functions for working with (whole-program) leak checking.
 
   // If heap leak checking is currently active in some mode
   // e.g. if leak checking was started (and is still active now)
-  // due to any valid non-empty --heap_check flag value
-  // (including "local") on the command-line
-  // or via a dependency on //base:heapcheck.
+  // due to HEAPCHECK=... defined in the environment.
   // The return value reflects iff HeapLeakChecker objects manually
   // constructed right now will be doing leak checking or nothing.
   // Note that we can go from active to inactive state during InitGoogle()
   // if FLAGS_heap_check gets set to "" by some code before/during InitGoogle().
   static bool IsActive();
 
-  // Return pointer to the whole-program checker if (still) active
+  // Return pointer to the whole-program checker if it has been created
   // and NULL otherwise.
+  // Once GlobalChecker() returns non-NULL that object will not disappear and
+  // will be returned by all later GlobalChecker calls.
   // This is mainly to access BytesLeaked() and ObjectsLeaked() (see below)
   // for the whole-program checker after one calls NoGlobalLeaks()
   // or similar and gets false.
@@ -92,7 +106,8 @@ class HeapLeakChecker {
   // has been called at least once on the whole-program checker.
   static void CancelGlobalCheck();
 
- public:  // Non-static functions for starting and doing leak checking.
+  // ----------------------------------------------------------------------- //
+  // Non-static functions for starting and doing leak checking.
 
   // Start checking and name the leak check performed.
   // The name is used in naming dumped profiles
@@ -101,6 +116,10 @@ class HeapLeakChecker {
   // in particular not contain path expressions.
   explicit HeapLeakChecker(const char *name);
 
+  // Destructor (verifies that some *NoLeaks or *SameHeap method
+  // has been called at least once).
+  ~HeapLeakChecker();
+
   // Return true iff the heap does not have more objects allocated
   // w.r.t. its state at the time of our construction.
   // This does full pprof heap change checking and reporting.
@@ -148,20 +167,8 @@ class HeapLeakChecker {
   ssize_t BytesLeaked() const;
   ssize_t ObjectsLeaked() const;
 
-  // Destructor (verifies that some *NoLeaks or *SameHeap method
-  // has been called at least once).
-  ~HeapLeakChecker();
-
- private:  // data
-
-  char* name_;  // our remembered name
-  size_t start_inuse_bytes_;  // bytes in use at our construction
-  size_t start_inuse_allocs_;  // allocations in use at our construction
-  bool has_checked_;  // if we have done the leak check, so these are ready:
-  ssize_t inuse_bytes_increase_;  // bytes-in-use increase for this checker
-  ssize_t inuse_allocs_increase_;  // allocations-in-use increase for this checker
-
- public:  // Static helpers to make us ignore certain leaks.
+  // ----------------------------------------------------------------------- //
+  // Static helpers to make us ignore certain leaks.
 
   // NOTE: All calls to DisableChecks* affect all later heap profile generation
   // that happens in our construction and inside of *NoLeaks().
@@ -200,7 +207,7 @@ class HeapLeakChecker {
   // because this disabling works by remembering the range of
   // return program counter addresses for the two calls.
   static void* GetDisableChecksStart();
-  static void DisableChecksToHereFrom(void* start_address);
+  static void DisableChecksToHereFrom(const void* start_address);
 
   // ADVICE: Use GetDisableChecksStart, DisableChecksToHereFrom
   //         instead of DisableChecksUp|At whenever possible
@@ -229,7 +236,7 @@ class HeapLeakChecker {
 
   // Same as DisableChecksUp,
   // but the function return program counter address is given explicitly.
-  static void DisableChecksAt(void* address);
+  static void DisableChecksAt(const void* address);
 
   // Tests for checking that DisableChecksUp and DisableChecksAt
   // behaved as expected, for example
@@ -247,65 +254,78 @@ class HeapLeakChecker {
   //    }
   // These return false when heap leak checking is turned off.
   static bool HaveDisabledChecksUp(int stack_frames);
-  static bool HaveDisabledChecksAt(void* address);
+  static bool HaveDisabledChecksAt(const void* address);
 
-  // Ignore an object located at 'ptr'
-  // (as well as all heap objects (transitively) referenced from it)
+  // Ignore an object located at 'ptr' (can go at the start or into the object)
+  // as well as all heap objects (transitively) referenced from it
   // for the purposes of heap leak checking.
   // If 'ptr' does not point to an active allocated object
   // at the time of this call, it is ignored;
   // but if it does, the object must not get deleted from the heap later on;
   // it must also be not already ignored at the time of this call.
-  static void IgnoreObject(void* ptr);
+  static void IgnoreObject(const void* ptr);
 
   // Undo what an earlier IgnoreObject() call promised and asked to do.
-  // At the time of this call 'ptr' must point to an active allocated object
-  // which was previously registered with IgnoreObject().
-  static void UnIgnoreObject(void* ptr);
+  // At the time of this call 'ptr' must point at or inside of an active
+  // allocated object which was previously registered with IgnoreObject().
+  static void UnIgnoreObject(const void* ptr);
 
- public:  // Initializations; to be called from main() only.
+  // ----------------------------------------------------------------------- //
+  // Initialization; to be called from main() only.
 
   // Full starting of recommended whole-program checking.
   static void InternalInitStart();
 
- public:  // Internal types defined in .cc
+  // ----------------------------------------------------------------------- //
+  // Internal types defined in .cc
 
-  struct Allocator;
+  class Allocator;
   struct RangeValue;
 
- private:  // Various helpers
+ private:
+
+  // ----------------------------------------------------------------------- //
+  // Various helpers
 
   // Type for DumpProfileLocked
   enum ProfileType { START_PROFILE, END_PROFILE };
+
   // Helper for dumping start/end heap leak checking profiles
   // and getting the byte/object counts.
-  void DumpProfileLocked(ProfileType profile_type, void* self_stack_top,
+  void DumpProfileLocked(ProfileType profile_type, const void* self_stack_top,
                          size_t* alloc_bytes, size_t* alloc_objects);
   // Helper for constructors
   void Create(const char *name);
+
   // Types for DoNoLeaks and its helpers.
   enum CheckType { SAME_HEAP, NO_LEAKS };
   enum CheckFullness { USE_PPROF, USE_COUNTS };
   enum ReportMode { PPROF_REPORT, NO_REPORT };
+
   // Helpers for *NoLeaks and *SameHeap
   bool DoNoLeaks(CheckType check_type,
                  CheckFullness fullness,
                  ReportMode report_mode);
-  bool DoNoLeaksOnce(CheckType check_type,
-                     CheckFullness fullness,
-                     ReportMode report_mode);
-  // Helper for IgnoreObject
-  static void IgnoreObjectLocked(void* ptr);
+  struct LeakCheckResult;  // defined in .cc
+  LeakCheckResult DoLeakCheckLocked();
+  bool DoNoLeaksOnceLocked(CheckType check_type,
+                           CheckFullness fullness,
+                           ReportMode report_mode);
+
   // Helper for DisableChecksAt
-  static void DisableChecksAtLocked(void* address);
+  static void DisableChecksAtLocked(const void* address);
+
   // Helper for DisableChecksIn
   static void DisableChecksInLocked(const char* pattern);
+
   // Helper for DisableChecksToHereFrom
-  static void DisableChecksFromToLocked(void* start_address,
-                                        void* end_address,
+  static void DisableChecksFromToLocked(const void* start_address,
+                                        const void* end_address,
                                         int max_depth);
+
   // Helper for DoNoLeaks to ignore all objects reachable from all live data
-  static void IgnoreAllLiveObjectsLocked(void* self_stack_top);
+  static void IgnoreAllLiveObjectsLocked(const void* self_stack_top);
+
   // Callback we pass to ListAllProcessThreads (see thread_lister.h)
   // that is invoked when all threads of our process are found and stopped.
   // The call back does the things needed to ignore live data reachable from
@@ -317,18 +337,20 @@ class HeapLeakChecker {
   // Here we only use num_threads and thread_pids, that ListAllProcessThreads
   // fills for us with the number and pids of all the threads of our process
   // it found and attached to.
-  static int IgnoreLiveThreads(void* parameter,
-                               int num_threads,
-                               pid_t* thread_pids,
-                               va_list ap);
-  // Helper for IgnoreAllLiveObjectsLocked and IgnoreLiveThreads
-  // that we prefer to execute from IgnoreLiveThreads
+  static int IgnoreLiveThreadsLocked(void* parameter,
+                                     int num_threads,
+                                     pid_t* thread_pids,
+                                     va_list ap);
+
+  // Helper for IgnoreAllLiveObjectsLocked and IgnoreLiveThreadsLocked
+  // that we prefer to execute from IgnoreLiveThreadsLocked
   // while all threads are stopped.
   // This helper does live object discovery and ignoring
   // for all objects that are reachable from everything
   // not related to thread stacks and registers.
   static void IgnoreNonThreadLiveObjectsLocked();
-  // Helper for IgnoreNonThreadLiveObjectsLocked and IgnoreLiveThreads
+
+  // Helper for IgnoreNonThreadLiveObjectsLocked and IgnoreLiveThreadsLocked
   // to discover and ignore all heap objects
   // reachable from currently considered live objects
   // (live_objects static global variable in out .cc file).
@@ -336,27 +358,31 @@ class HeapLeakChecker {
   // in a debug message to describe what kind of live object sources
   // are being used.
   static void IgnoreLiveObjectsLocked(const char* name, const char* name2);
-  // Heap profile object filtering callback to filter out live objects.
-  static bool HeapProfileFilter(void* ptr, size_t size);
+
   // Runs REGISTER_HEAPCHECK_CLEANUP cleanups and potentially
   // calls DoMainHeapCheck
   static void RunHeapCleanups();
-  // Do the overall whole-program heap leak check
-  static void DoMainHeapCheck();
+
+  // Do the overall whole-program heap leak check if needed;
+  // returns true when did the leak check.
+  static bool DoMainHeapCheck();
 
   // Type of task for UseProcMapsLocked
   enum ProcMapsTask {
     RECORD_GLOBAL_DATA,
     DISABLE_LIBRARY_ALLOCS
   };
+
   // Success/Error Return codes for UseProcMapsLocked.
   enum ProcMapsResult {
     PROC_MAPS_USED,
     CANT_OPEN_PROC_MAPS,
     NO_SHARED_LIBS_IN_PROC_MAPS
   };
+
   // Read /proc/self/maps, parse it, and do the 'proc_maps_task' for each line.
   static ProcMapsResult UseProcMapsLocked(ProcMapsTask proc_maps_task);
+
   // A ProcMapsTask to disable allocations from 'library'
   // that is mapped to [start_address..end_address)
   // (only if library is a certain system library).
@@ -364,36 +390,52 @@ class HeapLeakChecker {
                                          uintptr_t start_address,
                                          uintptr_t end_address);
 
-  // Return true iff "*ptr" points to a heap object;
-  // we also fill *object_size for this object then.
-  // If yes, we might move "*ptr" to point to the very start of the object
-  // (this needs to happen for C++ class array allocations
-  // and for basic_string-s of C++ library that comes with gcc 3.4).
-  static bool HaveOnHeapLocked(void** ptr, size_t* object_size);
+  // Return true iff "*ptr" points to a heap object
+  // ("*ptr" can point at the start or inside of a heap object
+  //  so that this works e.g. for pointers to C++ arrays, C++ strings,
+  //  multiple-inherited objects, or pointers to members).
+  // We also fill *object_size for this object then
+  // and we move "*ptr" to point to the very start of the heap object.
+  static inline bool HaveOnHeapLocked(const void** ptr, size_t* object_size);
 
- private:
+  // Helper to shutdown heap leak checker when it's not needed
+  // or can't function properly.
+  static void TurnItselfOffLocked();
+
+  // Internally-used c-tor to start whole-executable checking.
+  HeapLeakChecker();
+
+  // ----------------------------------------------------------------------- //
+  // Friends and externally accessed helpers.
 
   // Helper for VerifyHeapProfileTableStackGet in the unittest
   // to get the recorded allocation caller for ptr,
   // which must be a heap object.
-  static void* GetAllocCaller(void* ptr);
+  static const void* GetAllocCaller(void* ptr);
   friend void VerifyHeapProfileTableStackGet();
 
   // This gets to execute before constructors for all global objects
-  static void BeforeConstructors();
+  static void BeforeConstructorsLocked();
   friend void HeapLeakChecker_BeforeConstructors();
+
   // This gets to execute after destructors for all global objects
   friend void HeapLeakChecker_AfterDestructors();
-  // Helper to shutdown heap leak checker when it's not needed
-  // or can't function properly.
-  static void TurnItselfOff();
 
- private:
+  // ----------------------------------------------------------------------- //
+  // Member data.
 
-  // Start whole-executable checking.
-  HeapLeakChecker();
+  class SpinLock* lock_;  // to make HeapLeakChecker objects thread-safe
+  const char* name_;  // our remembered name (we own it)
+                      // NULL means this leak checker is a noop
+  size_t start_inuse_bytes_;  // bytes in use at our construction
+  size_t start_inuse_allocs_;  // allocations in use at our construction
+  bool has_checked_;  // if we have done the leak check, so these are ready:
+  ssize_t inuse_bytes_increase_;  // bytes-in-use increase for this checker
+  ssize_t inuse_allocs_increase_;  // allocations-in-use increase
+                                   // for this checker
+
+  // ----------------------------------------------------------------------- //
 
- private:
   // Disallow "evil" constructors.
   HeapLeakChecker(const HeapLeakChecker&);
   void operator=(const HeapLeakChecker&);
@@ -402,7 +444,7 @@ class HeapLeakChecker {
 
 // A class that exists solely to run its destructor.  This class should not be
 // used directly, but instead by the REGISTER_HEAPCHECK_CLEANUP macro below.
-class HeapCleaner {
+class PERFTOOLS_DLL_DECL HeapCleaner {
  public:
   typedef void (*void_function)(void);
   HeapCleaner(void_function f);
@@ -421,4 +463,4 @@ class HeapCleaner {
   static HeapCleaner heapcheck_cleaner_##name(&heapcheck_cleanup_##name); \
   }
 
-#endif  // BASE_HEAP_CHECKER_H__
+#endif  // BASE_HEAP_CHECKER_H_
