@@ -35,16 +35,27 @@
 // application can link against a malloc that does not implement these
 // extensions, and it will get default versions that do nothing.
 
-#ifndef BASE_MALLOC_EXTENSION_H__
-#define BASE_MALLOC_EXTENSION_H__
+#ifndef BASE_MALLOC_EXTENSION_H_
+#define BASE_MALLOC_EXTENSION_H_
 
 #include <stddef.h>
 #include <string>
 
+// Annoying stuff for windows -- makes sure clients can import these functions
+#ifndef PERFTOOLS_DLL_DECL
+# ifdef WIN32
+#   define PERFTOOLS_DLL_DECL  __declspec(dllimport)
+# else
+#   define PERFTOOLS_DLL_DECL
+# endif
+#endif
+
 static const int kMallocHistogramSize = 64;
 
 // The default implementations of the following routines do nothing.
-class MallocExtension {
+// All implementations should be thread-safe; the current one
+// (TCMallocImplementation) is.
+class PERFTOOLS_DLL_DECL MallocExtension {
  public:
   virtual ~MallocExtension();
 
@@ -190,4 +201,4 @@ class MallocExtension {
   virtual void** ReadHeapGrowthStackTraces();
 };
 
-#endif  // BASE_MALLOC_EXTENSION_H__
+#endif  // BASE_MALLOC_EXTENSION_H_
