@@ -30,34 +30,73 @@
 // ---
 // Author: Sanjay Ghemawat
 //
-// Routine to extract the current stack trace.
+// Routines to extract the current stack trace.  These functions are
+// thread-safe.
 
-#ifndef _GOOGLE_STACKTRACE_H
-#define _GOOGLE_STACKTRACE_H
+#ifndef GOOGLE_STACKTRACE_H_
+#define GOOGLE_STACKTRACE_H_
 
-// Skip the most recent "skip_count" stack frames (also skips the
-// frame generated for the "GetStackTrace" routine itself), and then
-// record the pc values for upto the next "max_depth" frames in
-// "result".  Returns the number of values recorded in "result".
+// Annoying stuff for windows -- makes sure clients can import these functions
+#ifndef PERFTOOLS_DLL_DECL
+# ifdef WIN32
+#   define PERFTOOLS_DLL_DECL  __declspec(dllimport)
+# else
+#   define PERFTOOLS_DLL_DECL
+# endif
+#endif
+
+
+// Skips the most recent "skip_count" stack frames (also skips the
+// frame generated for the "GetStackFrames" routine itself), and then
+// records the pc values for up to the next "max_depth" frames in
+// "pcs", and the corresponding stack frame sizes in "sizes".  Returns
+// the number of values recorded in "pcs"/"sizes".
 //
 // Example:
 //      main() { foo(); }
 //      foo() { bar(); }
 //      bar() {
-//        void* result[10];
-//        int depth = GetStackTrace(result, 10, 1);
+//        void* pcs[10];
+//        int sizes[10];
+//        int depth = GetStackFrames(pcs, sizes, 10, 1);
 //      }
 //
-// The GetStackTrace call will skip the frame for "bar".  It will
+// The GetStackFrames call will skip the frame for "bar".  It will
 // return 2 and will produce pc values that map to the following
 // procedures:
-//      result[0]       foo
-//      result[1]       main
+//      pcs[0]       foo
+//      pcs[1]       main
 // (Actually, there may be a few more entries after "main" to account for
 // startup procedures.)
+// And corresponding stack frame sizes will also be recorded:
+//    sizes[0]       16
+//    sizes[1]       16
+// (Stack frame sizes of 16 above are just for illustration purposes.)
+// Stack frame sizes of 0 or less indicate that those frame sizes couldn't
+// be identified.
 //
-// This routine currently produces non-empty stack traces only for
-// Linux/x86 machines.
-extern int GetStackTrace(void** result, int max_depth, int skip_count);
+// This routine may return fewer stack frame entries than are
+// available. Also note that "pcs" and "sizes" must both be non-NULL.
+extern PERFTOOLS_DLL_DECL int GetStackFrames(void** pcs, int* sizes, int max_depth,
+                          int skip_count);
 
-#endif /* _GOOGLE_STACKTRACE_H */
+// This is similar to the GetStackFrames routine, except that it returns
+// the stack trace only, and not the stack frame sizes as well.
+// Example:
+//      main() { foo(); }
+//      foo() { bar(); }
+//      bar() {
+//        void* result[10];
+//        int depth = GetStackFrames(result, 10, 1);
+//      }
+//
+// This produces:
+//      result[0]       foo
+//      result[1]       main
+//           ....       ...
+//
+// "result" must not be NULL.
+extern PERFTOOLS_DLL_DECL int GetStackTrace(void** result, int max_depth,
+                                            int skip_count);
+
+#endif /* GOOGLE_STACKTRACE_H_ */
