@@ -35,9 +35,12 @@
 #include "config.h"
 
 #include <stdlib.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
-#include <fcntl.h>
+#endif
+#ifdef HAVE_MMAP
 #include <sys/mman.h>
+#endif
 #include <errno.h>
 #include <assert.h>
 
@@ -50,11 +53,14 @@
 #include "base/basictypes.h"   // for PRId64, among other things
 #include "base/googleinit.h"
 #include "base/commandlineflags.h"
+#include "malloc_hook-inl.h"
 #include <google/malloc_hook.h>
 #include <google/malloc_extension.h>
 #include "base/spinlock.h"
 #include "base/low_level_alloc.h"
+#include "base/sysinfo.h"      // for GetUniquePathFromEnv()
 #include "heap-profile-table.h"
+#include "memory_region_map.h"
 
 
 #ifndef	PATH_MAX
@@ -70,6 +76,9 @@ using STL_NAMESPACE::sort;
 
 //----------------------------------------------------------------------
 // Flags that control heap-profiling
+//
+// The thread-safety of the profiler depends on these being immutable
+// after main starts, so don't change them.
 //----------------------------------------------------------------------
 
 DEFINE_int64(heap_profile_allocation_interval,
@@ -86,7 +95,12 @@ DEFINE_bool(mmap_log,
             "Should mmap/munmap calls be logged?");
 DEFINE_bool(mmap_profile,
             EnvToBool("HEAP_PROFILE_MMAP", false),
-            "If heap-profiling on, also profile mmaps");
+            "If heap-profiling is on, also profile mmap, mremap, and sbrk)");
+DEFINE_bool(only_mmap_profile,
+            EnvToBool("HEAP_PROFILE_ONLY_MMAP", false),
+            "If heap-profiling is on, only profile mmap, mremap, and sbrk; "
+            "do not profile malloc/new/etc");
+
 
 //----------------------------------------------------------------------
 // Locking
@@ -107,7 +121,7 @@ static SpinLock heap_lock(SpinLock::LINKER_INITIALIZED);
 static LowLevelAlloc::Arena *heap_profiler_memory;
 
 static void* ProfilerMalloc(size_t bytes) {
-  return LowLevelAlloc::Alloc(bytes, heap_profiler_memory);
+  return LowLevelAlloc::AllocWithArena(bytes, heap_profiler_memory);
 }
 static void ProfilerFree(void* p) {
   LowLevelAlloc::Free(p);
@@ -131,32 +145,84 @@ static HeapProfileTable* heap_profile = NULL;  // the heap profile table
 // Profile generation
 //----------------------------------------------------------------------
 
-char* GetHeapProfile() {
+enum AddOrRemove { ADD, REMOVE };
+
+// Add or remove all MMap-allocated regions to/from *heap_profile.
+// Assumes heap_lock is held.
+static void AddRemoveMMapDataLocked(AddOrRemove mode) {
+  RAW_DCHECK(heap_lock.IsHeld(), "");
+  if (!FLAGS_mmap_profile || !is_on) return;
+  if (!FLAGS_mmap_log) MemoryRegionMap::CheckMallocHooks();
+  // MemoryRegionMap maintained all the data we need for all
+  // mmap-like allocations, so we just use it here:
+  MemoryRegionMap::LockHolder l;
+  for (MemoryRegionMap::RegionIterator r = MemoryRegionMap::BeginRegionLocked();
+       r != MemoryRegionMap::EndRegionLocked(); ++r) {
+    if (mode == ADD) {
+      heap_profile->RecordAllocWithStack(
+        reinterpret_cast<const void*>(r->start_addr),
+        r->end_addr - r->start_addr,
+        r->call_stack_depth, r->call_stack);
+    } else {
+      heap_profile->RecordFree(reinterpret_cast<void*>(r->start_addr));
+    }
+  }
+}
+
+static char* DoGetHeapProfile(void* (*alloc_func)(size_t)) {
   // We used to be smarter about estimating the required memory and
   // then capping it to 1MB and generating the profile into that.
   // However it should not cost us much to allocate 1MB every time.
   static const int size = 1 << 20;
-  // This is intended to be normal malloc: we return it to the user to free it
-  char* buf = reinterpret_cast<char*>(malloc(size));
+  char* buf = reinterpret_cast<char*>((*alloc_func)(size));
   if (buf == NULL) return NULL;
 
   // Grab the lock and generate the profile.
-  heap_lock.Lock();
-  int buflen = is_on ? heap_profile->FillOrderedProfile(buf, size - 1) : 0;
+  SpinLockHolder l(&heap_lock);
+  int buflen = 0;
+  if (is_on) {
+    HeapProfileTable::Stats const stats = heap_profile->total();
+    AddRemoveMMapDataLocked(ADD);
+    buflen = heap_profile->FillOrderedProfile(buf, size - 1);
+    // FillOrderedProfile should not reduce the set of active mmap-ed regions,
+    // hence MemoryRegionMap will let us remove everything we've added above:
+    AddRemoveMMapDataLocked(REMOVE);
+    RAW_DCHECK(stats.Equivalent(heap_profile->total()), "");
+    // if this fails, we somehow removed by AddRemoveMMapDataLocked
+    // more than we have added.
+  }
   buf[buflen] = '\0';
   RAW_DCHECK(buflen == strlen(buf), "");
-  heap_lock.Unlock();
 
   return buf;
 }
 
+extern "C" char* GetHeapProfile() {
+  // Use normal malloc: we return the profile to the user to free it:
+  return DoGetHeapProfile(&malloc);
+}
+
+// defined below
+static void NewHook(const void* ptr, size_t size);
+static void DeleteHook(const void* ptr);
+
 // Helper for HeapProfilerDump.
 static void DumpProfileLocked(const char* reason) {
+  RAW_DCHECK(heap_lock.IsHeld(), "");
   RAW_DCHECK(is_on, "");
   RAW_DCHECK(!dumping, "");
-  
+
   if (filename_prefix == NULL) return;  // we do not yet need dumping
 
+  if (FLAGS_only_mmap_profile == false) {
+    if (MallocHook::GetNewHook() != NewHook  ||
+        MallocHook::GetDeleteHook() != DeleteHook) {
+      RAW_LOG(FATAL, "Had our new/delete MallocHook-s replaced. "
+                     "Are you using another MallocHook client? "
+                     "Do not use --heap_profile=... to avoid this conflict.");
+    }
+  }
+
   dumping = true;
 
   // Make file name
@@ -173,9 +239,10 @@ static void DumpProfileLocked(const char* reason) {
     RAW_VLOG(0, "Dumping heap profile to %s (%s)", file_name, reason);
     FILE* f = fopen(file_name, "w");
     if (f != NULL) {
-      const char* profile = GetHeapProfile();
+      // Use ProfilerMalloc not to put info about profile itself into itself:
+      char* profile = DoGetHeapProfile(&ProfilerMalloc);
       fputs(profile, f);
-      free(const_cast<char*>(profile));  // was made with normal malloc
+      ProfilerFree(profile);
       fclose(f);
     } else {
       RAW_LOG(ERROR, "Failed dumping heap profile to %s", file_name);
@@ -191,8 +258,8 @@ static void DumpProfileLocked(const char* reason) {
 //----------------------------------------------------------------------
 
 // Record an allocation in the profile.
-static void RecordAlloc(void* ptr, size_t bytes, int skip_count) {
-  heap_lock.Lock();
+static void RecordAlloc(const void* ptr, size_t bytes, int skip_count) {
+  SpinLockHolder l(&heap_lock);
   if (is_on) {
     heap_profile->RecordAlloc(ptr, bytes, skip_count + 1);
     const HeapProfileTable::Stats& total = heap_profile->total();
@@ -209,7 +276,7 @@ static void RecordAlloc(void* ptr, size_t bytes, int skip_count) {
         need_to_dump = true;
       } else if (inuse_bytes >
                  high_water_mark + FLAGS_heap_profile_inuse_interval) {
-        sprintf(buf, "%"PRId64" MB in use", inuse_bytes >> 20);
+        snprintf(buf, sizeof(buf), "%"PRId64" MB in use", inuse_bytes >> 20);
         // Track that we made a "high water mark" dump
         high_water_mark = inuse_bytes;
         need_to_dump = true;
@@ -219,67 +286,113 @@ static void RecordAlloc(void* ptr, size_t bytes, int skip_count) {
       }
     }
   }
-  heap_lock.Unlock();
 }
 
 // Record a deallocation in the profile.
-static void RecordFree(void* ptr) {
-  heap_lock.Lock();
+static void RecordFree(const void* ptr) {
+  SpinLockHolder l(&heap_lock);
   if (is_on) {
     heap_profile->RecordFree(ptr);
   }
-  heap_lock.Unlock();
 }
 
-
 //----------------------------------------------------------------------
 // Allocation/deallocation hooks for MallocHook
 //----------------------------------------------------------------------
 
-static void NewHook(void* ptr, size_t size) {
+// static
+void NewHook(const void* ptr, size_t size) {
   if (ptr != NULL) RecordAlloc(ptr, size, 0);
 }
 
-static void DeleteHook(void* ptr) {
+// static
+void DeleteHook(const void* ptr) {
   if (ptr != NULL) RecordFree(ptr);
 }
 
-static void MmapHook(void* result, void* start, size_t size,
+// TODO(jandrews): Re-enable stack tracing
+#ifdef TODO_REENABLE_STACK_TRACING
+static void RawInfoStackDumper(const char* message, void*) {
+  RAW_LOG(INFO, "%.*s", static_cast<int>(strlen(message) - 1), message);
+  // -1 is to chop the \n which will be added by RAW_LOG
+}
+#endif
+
+// Saved MemoryRegionMap's hooks to daisy-chain calls to.
+MallocHook::MmapHook saved_mmap_hook = NULL;
+MallocHook::MremapHook saved_mremap_hook = NULL;
+MallocHook::MunmapHook saved_munmap_hook = NULL;
+MallocHook::SbrkHook saved_sbrk_hook = NULL;
+
+static void MmapHook(const void* result, const void* start, size_t size,
                      int prot, int flags, int fd, off_t offset) {
-  // Log the mmap if necessary
-  if (FLAGS_mmap_log) {
-    char buf[200];
+  if (FLAGS_mmap_log) {  // log it
     // We use PRIxS not just '%p' to avoid deadlocks
     // in pretty-printing of NULL as "nil".
     // TODO(maxim): instead should use a safe snprintf reimplementation
-    snprintf(buf, sizeof(buf),
-             "mmap(start=0x%"PRIxS", len=%"PRIuS", prot=0x%x, flags=0x%x, "
-             "fd=%d, offset=0x%x) = 0x%"PRIxS"",
-             (uintptr_t) start, size, prot, flags, fd, (unsigned int) offset,
-             (uintptr_t) result);
-    LOG(INFO, "%s", buf);
-    // TODO(jandrews): Re-enable stack tracing
-    //DumpStackTrace(1, DebugWriteToStream, &LOG(INFO));
+    RAW_LOG(INFO,
+            "mmap(start=0x%"PRIxS", len=%"PRIuS", prot=0x%x, flags=0x%x, "
+            "fd=%d, offset=0x%x) = 0x%"PRIxS"",
+            (uintptr_t) start, size, prot, flags, fd, (unsigned int) offset,
+            (uintptr_t) result);
+#ifdef TODO_REENABLE_STACK_TRACING
+    DumpStackTrace(1, RawInfoStackDumper, NULL);
+#endif
   }
-
-  // Record mmap in profile if appropriate
-  if (FLAGS_mmap_profile && result != (void*) MAP_FAILED) {
-    RecordAlloc(result, size, 0);
+  if (saved_mmap_hook) {
+    // Call MemoryRegionMap's hook: it will record needed info about the mmap
+    // for us w/o deadlocks:
+    (*saved_mmap_hook)(result, start, size, prot, flags, fd, offset);
   }
 }
 
-static void MunmapHook(void* ptr, size_t size) {
-  if (FLAGS_mmap_profile) {
-    RecordFree(ptr);
-  }
-  if (FLAGS_mmap_log) {
-    char buf[200];
+static void MremapHook(const void* result, const void* old_addr,
+                       size_t old_size, size_t new_size,
+                       int flags, const void* new_addr) {
+  if (FLAGS_mmap_log) {  // log it
     // We use PRIxS not just '%p' to avoid deadlocks
     // in pretty-printing of NULL as "nil".
     // TODO(maxim): instead should use a safe snprintf reimplementation
-    snprintf(buf, sizeof(buf),
-             "munmap(start=0x%"PRIxS", len=%"PRIuS")", (uintptr_t) ptr, size);
-    LOG(INFO, "%s", buf);
+    RAW_LOG(INFO,
+            "mremap(old_addr=0x%"PRIxS", old_size=%"PRIuS", new_size=%"PRIuS", "
+            "flags=0x%x, new_addr=0x%"PRIxS") = 0x%"PRIxS"",
+            (uintptr_t) old_addr, old_size, new_size, flags,
+            (uintptr_t) new_addr, (uintptr_t) result);
+#ifdef TODO_REENABLE_STACK_TRACING
+    DumpStackTrace(1, RawInfoStackDumper, NULL);
+#endif
+  }
+  if (saved_mremap_hook) {  // call MemoryRegionMap's hook
+    (*saved_mremap_hook)(result, old_addr, old_size, new_size, flags, new_addr);
+  }
+}
+
+static void MunmapHook(const void* ptr, size_t size) {
+  if (FLAGS_mmap_log) {  // log it
+    // We use PRIxS not just '%p' to avoid deadlocks
+    // in pretty-printing of NULL as "nil".
+    // TODO(maxim): instead should use a safe snprintf reimplementation
+    RAW_LOG(INFO, "munmap(start=0x%"PRIxS", len=%"PRIuS")",
+                  (uintptr_t) ptr, size);
+#ifdef TODO_REENABLE_STACK_TRACING
+    DumpStackTrace(1, RawInfoStackDumper, NULL);
+#endif
+  }
+  if (saved_munmap_hook) {  // call MemoryRegionMap's hook
+    (*saved_munmap_hook)(ptr, size);
+  }
+}
+
+static void SbrkHook(const void* result, ptrdiff_t increment) {
+  if (FLAGS_mmap_log) {  // log it
+    RAW_LOG(INFO, "sbrk(inc=%"PRIdS") = 0x%"PRIxS"",
+                  increment, (uintptr_t) result);
+#ifdef TODO_REENABLE_STACK_TRACING
+    DumpStackTrace(1, RawInfoStackDumper, NULL);
+#endif
+  }
+  if (saved_sbrk_hook) {  // call MemoryRegionMap's hook
+    (*saved_sbrk_hook)(result, increment);
   }
 }
 
@@ -287,59 +400,100 @@ static void MunmapHook(void* ptr, size_t size) {
 // Starting/stopping/dumping
 //----------------------------------------------------------------------
 
-void HeapProfilerStart(const char* prefix) {
-  heap_lock.Lock();
+extern "C" void HeapProfilerStart(const char* prefix) {
+  SpinLockHolder l(&heap_lock);
 
-  if (filename_prefix != NULL) return;
-
-  RAW_DCHECK(!is_on, "");
-
-  heap_profiler_memory = LowLevelAlloc::NewArena(0, 0);
-
-  heap_profile = new (ProfilerMalloc(sizeof(HeapProfileTable)))
-                   HeapProfileTable(ProfilerMalloc, ProfilerFree);
+  if (is_on) return;
 
   is_on = true;
 
+  RAW_VLOG(0, "Starting tracking the heap");
+
+  if (FLAGS_only_mmap_profile) {
+    FLAGS_mmap_profile = true;
+  }
+
+  if (FLAGS_mmap_profile) {
+    // Ask MemoryRegionMap to record all mmap, mremap, and sbrk
+    // call stack traces of at least size kMaxStackDepth:
+    MemoryRegionMap::Init(HeapProfileTable::kMaxStackDepth);
+  }
+
+  if (FLAGS_mmap_log) {
+    // Install our hooks to do the logging
+    // and maybe save MemoryRegionMap's hooks to call:
+    saved_mmap_hook = MallocHook::SetMmapHook(MmapHook);
+    saved_mremap_hook = MallocHook::SetMremapHook(MremapHook);
+    saved_munmap_hook = MallocHook::SetMunmapHook(MunmapHook);
+    saved_sbrk_hook = MallocHook::SetSbrkHook(SbrkHook);
+  }
+
+  heap_profiler_memory =
+    LowLevelAlloc::NewArena(0, LowLevelAlloc::DefaultArena());
+
+  heap_profile = new(ProfilerMalloc(sizeof(HeapProfileTable)))
+                   HeapProfileTable(ProfilerMalloc, ProfilerFree);
+
   last_dump = 0;
 
   // We do not reset dump_count so if the user does a sequence of
   // HeapProfilerStart/HeapProfileStop, we will get a continuous
   // sequence of profiles.
 
-  // Now set the hooks that capture mallocs/frees
-  MallocHook::SetNewHook(NewHook);
-  MallocHook::SetDeleteHook(DeleteHook);
-  RAW_VLOG(0, "Starting tracking the heap");
+  if (FLAGS_only_mmap_profile == false) {
+    // Now set the hooks that capture new/delete and malloc/free
+    // and check that these are the only hooks:
+    if (MallocHook::SetNewHook(NewHook) != NULL  ||
+        MallocHook::SetDeleteHook(DeleteHook) != NULL) {
+      RAW_LOG(FATAL, "Had other new/delete MallocHook-s set. "
+                     "Are you using the heap leak checker? "
+                     "Use --heap_check=\"\" to avoid this conflict.");
+    }
+  }
 
   // Copy filename prefix
+  RAW_DCHECK(filename_prefix == NULL, "");
   const int prefix_length = strlen(prefix);
   filename_prefix = reinterpret_cast<char*>(ProfilerMalloc(prefix_length + 1));
   memcpy(filename_prefix, prefix, prefix_length);
   filename_prefix[prefix_length] = '\0';
 
-  heap_lock.Unlock();
-
   // This should be done before the hooks are set up, since it should
   // call new, and we want that to be accounted for correctly.
   MallocExtension::Initialize();
 }
 
-void HeapProfilerStop() {
-  heap_lock.Lock();
+extern "C" void HeapProfilerStop() {
+  SpinLockHolder l(&heap_lock);
 
   if (!is_on) return;
 
-  filename_prefix = NULL;
-
-  MallocHook::SetNewHook(NULL);
-  MallocHook::SetDeleteHook(NULL);
+  if (FLAGS_only_mmap_profile == false) {
+    // Unset our new/delete hooks, checking they were the ones set:
+    if (MallocHook::SetNewHook(NULL) != NewHook  ||
+        MallocHook::SetDeleteHook(NULL) != DeleteHook) {
+      RAW_LOG(FATAL, "Had our new/delete MallocHook-s replaced. "
+                     "Are you using another MallocHook client? "
+                     "Do not use --heap_profile=... to avoid this conflict.");
+    }
+  }
+  if (FLAGS_mmap_log) {
+    // Restore mmap/sbrk hooks, checking that our hooks were the ones set:
+    if (MallocHook::SetMmapHook(saved_mmap_hook) != MmapHook  ||
+        MallocHook::SetMremapHook(saved_mremap_hook) != MremapHook  ||
+        MallocHook::SetMunmapHook(saved_munmap_hook) != MunmapHook  ||
+        MallocHook::SetSbrkHook(saved_sbrk_hook) != SbrkHook) {
+      RAW_LOG(FATAL, "Had our mmap/mremap/munmap/sbrk MallocHook-s replaced. "
+                     "Are you using another MallocHook client? "
+                     "Do not use --heap_profile=... to avoid this conflict.");
+    }
+  }
 
   // free profile
   heap_profile->~HeapProfileTable();
   ProfilerFree(heap_profile);
   heap_profile = NULL;
-  
+
   // free prefix
   ProfilerFree(filename_prefix);
   filename_prefix = NULL;
@@ -348,17 +502,18 @@ void HeapProfilerStop() {
     RAW_LOG(FATAL, "Memory leak in HeapProfiler:");
   }
 
-  is_on = false;
+  if (FLAGS_mmap_profile) {
+    MemoryRegionMap::Shutdown();
+  }
 
-  heap_lock.Unlock();
+  is_on = false;
 }
 
-void HeapProfilerDump(const char *reason) {
-  heap_lock.Lock();
+extern "C" void HeapProfilerDump(const char *reason) {
+  SpinLockHolder l(&heap_lock);
   if (is_on && !dumping) {
     DumpProfileLocked(reason);
   }
-  heap_lock.Unlock();
 }
 
 //----------------------------------------------------------------------
@@ -367,47 +522,19 @@ void HeapProfilerDump(const char *reason) {
 
 // Initialization code
 static void HeapProfilerInit() {
-  if (FLAGS_mmap_profile || FLAGS_mmap_log) {
-    MallocHook::SetMmapHook(MmapHook);
-    MallocHook::SetMunmapHook(MunmapHook);
-  }
-
   // Everything after this point is for setting up the profiler based on envvar
-
-  char* heapprofile = getenv("HEAPPROFILE");
-  if (!heapprofile || heapprofile[0] == '\0') {
+  char fname[PATH_MAX];
+  if (!GetUniquePathFromEnv("HEAPPROFILE", fname)) {
     return;
   }
   // We do a uid check so we don't write out files in a setuid executable.
+#ifdef HAVE_GETEUID
   if (getuid() != geteuid()) {
     RAW_LOG(WARNING, ("HeapProfiler: ignoring HEAPPROFILE because "
                       "program seems to be setuid\n"));
     return;
   }
-
-  // If we're a child process of the 'main' process, we can't just use
-  // the name HEAPPROFILE -- the parent process will be using that.
-  // Instead we append our pid to the name.  How do we tell if we're a
-  // child process?  Ideally we'd set an environment variable that all
-  // our children would inherit.  But -- and perhaps this is a bug in
-  // gcc -- if you do a setenv() in a shared libarary in a global
-  // constructor, the environment setting is lost by the time main()
-  // is called.  The only safe thing we can do in such a situation is
-  // to modify the existing envvar.  So we do a hack: in the parent,
-  // we set the high bit of the 1st char of HEAPPROFILE.  In the child,
-  // we notice the high bit is set and append the pid().  This works
-  // assuming cpuprofile filenames don't normally have the high bit
-  // set in their first character!  If that assumption is violated,
-  // we'll still get a profile, but one with an unexpected name.
-  // TODO(csilvers): set an envvar instead when we can do it reliably.
-  char fname[PATH_MAX];
-  if (heapprofile[0] & 128) {                   // high bit is set
-    snprintf(fname, sizeof(fname), "%c%s_%u",   // add pid and clear high bit
-             heapprofile[0] & 127, heapprofile+1, (unsigned int)(getpid()));
-  } else {
-    snprintf(fname, sizeof(fname), "%s", heapprofile);
-    heapprofile[0] |= 128;                      // set high bit for kids to see
-  }
+#endif
 
   HeapProfileTable::CleanupOldProfiles(fname);
 
