@@ -32,13 +32,11 @@
 //
 // Internal logging and related utility routines.
 
-#ifndef TCMALLOC_INTERNAL_LOGGING_H__
-#define TCMALLOC_INTERNAL_LOGGING_H__
+#ifndef TCMALLOC_INTERNAL_LOGGING_H_
+#define TCMALLOC_INTERNAL_LOGGING_H_
 
 #include "config.h"
-#include <stdarg.h>
-#include <stdlib.h>
-#include <unistd.h>
+#include <stdlib.h>   // for abort()
 
 //-------------------------------------------------------------------
 // Utility routines
@@ -56,13 +54,21 @@ extern void TCMalloc_MESSAGE(const char* format, ...)
 // Short form for convenience
 #define MESSAGE TCMalloc_MESSAGE
 
+// Dumps the specified message and then calls abort()
+extern void TCMalloc_CRASH(const char* format, ...)
+#ifdef HAVE___ATTRIBUTE__
+  __attribute__ ((__format__ (__printf__, 1, 2)))
+#endif
+;
+
+#define CRASH TCMalloc_CRASH
+
 // Like assert(), but executed even in NDEBUG mode
 #undef CHECK_CONDITION
 #define CHECK_CONDITION(cond)                                            \
 do {                                                                     \
   if (!(cond)) {                                                         \
-    MESSAGE("%s:%d: assertion failed: %s\n", __FILE__, __LINE__, #cond); \
-    abort();                                                             \
+    CRASH("%s:%d: assertion failed: %s\n", __FILE__, __LINE__, #cond);   \
   }                                                                      \
 } while (0)
 
@@ -93,4 +99,4 @@ class TCMalloc_Printer {
 ;
 };
 
-#endif  // TCMALLOC_INTERNAL_LOGGING_H__
+#endif  // TCMALLOC_INTERNAL_LOGGING_H_
