@@ -33,7 +33,6 @@
 #include "config.h"
 #include <assert.h>
 #include <string.h>
-#include <pthread.h>
 #include <stdio.h>
 #if defined HAVE_STDINT_H
 #include <stdint.h>
@@ -43,7 +42,7 @@
 #include <sys/types.h>
 #endif
 #include <string>
-#include "google/perftools/hash_set.h"
+#include HASH_SET_H          // defined in config.h
 #include "google/malloc_extension.h"
 #include "maybe_threads.h"
 
@@ -56,6 +55,7 @@ void MallocExtension::Initialize() {
   if (initialize_called) return;
   initialize_called = true;
 
+#ifdef __GLIBC__
   // GNU libc++ versions 3.3 and 3.4 obey the environment variables
   // GLIBCPP_FORCE_NEW and GLIBCXX_FORCE_NEW respectively.  Setting
   // one of these variables forces the STL default allocator to call
@@ -74,6 +74,7 @@ void MallocExtension::Initialize() {
   // caches what it finds.  So we just cause an stl alloc here.
   string dummy("I need to be allocated");
   dummy += "!";         // so the definition of dummy isn't optimized out
+#endif  /* __GLIBC__ */
 }
 
 // Default implementation -- does nothing
@@ -100,7 +101,7 @@ bool MallocExtension::MallocMemoryStats(int* blocks, size_t* total,
                                        int histogram[kMallocHistogramSize]) {
   *blocks = 0;
   *total = 0;
-  memset(histogram, sizeof(histogram), 0);
+  memset(histogram, 0, sizeof(histogram));
   return true;
 }
 
@@ -205,7 +206,22 @@ struct StackTraceEqual {
   }
 };
 
+#ifdef WIN32
+typedef HASH_NAMESPACE::hash_set<void**, StackTraceHash> StackTraceTable;
+#else
 typedef HASH_NAMESPACE::hash_set<void**, StackTraceHash, StackTraceEqual> StackTraceTable;
+#endif
+
+void PrintCountAndSize(string* result, uintptr_t count, uintptr_t size) {
+  char buf[100];
+  snprintf(buf, sizeof(buf),
+           "%6lld: %8lld [%6lld: %8lld] @",
+           static_cast<long long>(count),
+           static_cast<long long>(size),
+           static_cast<long long>(count),
+           static_cast<long long>(size));
+  *result += buf;
+}
 
 void PrintHeader(string* result, const char* label, void** entries) {
   // Compute the total count and total size
@@ -216,24 +232,16 @@ void PrintHeader(string* result, const char* label, void** entries) {
     total_size += Size(entry);
   }
 
-  char buf[200];
-  snprintf(buf, sizeof(buf),
-           "heap profile: %6lld: %8lld [%6lld: %8lld] @ %s\n",
-           static_cast<long long>(total_count),
-           static_cast<long long>(total_size),
-           static_cast<long long>(total_count),
-           static_cast<long long>(total_size),
-           label);
-  *result += buf;
+  *result += string("heap profile: ");
+  PrintCountAndSize(result, total_count, total_size);
+  *result += string(" ") + label + "\n";
 }
 
 void PrintStackEntry(string* result, void** entry) {
-  char buf[100];
-  snprintf(buf, sizeof(buf), "%6d: %8d [%6d: %8d] @",
-           int(Count(entry)), int(Size(entry)),
-           int(Count(entry)), int(Size(entry)));
-  *result += buf;
+  PrintCountAndSize(result, Count(entry), Size(entry));
+
   for (int i = 0; i < Depth(entry); i++) {
+    char buf[32];
     snprintf(buf, sizeof(buf), " %p", PC(entry, i));
     *result += buf;
   }
