@@ -34,14 +34,18 @@
 
 // Disable the glibc prototype of mremap(), as older versions of the
 // system headers define this function with only four arguments,
-// whereas newer versions allow an optional fifth argument: 
-#define mremap glibc_mremap
-#include <sys/mman.h>
-#undef mremap
+// whereas newer versions allow an optional fifth argument:
+#ifdef HAVE_MMAP
+# define mremap glibc_mremap
+# include <sys/mman.h>
+# undef mremap
+#endif
 
-#include <google/malloc_hook.h>
+#include <algorithm>
 #include "base/basictypes.h"
 #include "base/logging.h"
+#include "malloc_hook-inl.h"
+#include <google/malloc_hook.h>
 #include <google/stacktrace.h>
 
 // __THROW is defined in glibc systems.  It means, counter-intuitively,
@@ -51,6 +55,9 @@
 # define __THROW   // __THROW is just an optimization, so ok to make it ""
 #endif
 
+using std::copy;
+
+
 // Declarations of three default weak hook functions, that can be overridden by
 // linking-in a strong definition (as heap-checker.cc does)
 //
@@ -66,11 +73,11 @@
 // (weak) here, then defining it below after its use, we can avoid the problem.
 //
 ATTRIBUTE_WEAK
-extern void InitialMallocHook_New(void* ptr, size_t size);
+extern void InitialMallocHook_New(const void* ptr, size_t size);
 
 ATTRIBUTE_WEAK
-extern void InitialMallocHook_MMap(void* result,
-                                   void* start,
+extern void InitialMallocHook_MMap(const void* result,
+                                   const void* start,
                                    size_t size,
                                    int protection,
                                    int flags,
@@ -78,14 +85,74 @@ extern void InitialMallocHook_MMap(void* result,
                                    off_t offset);
 
 ATTRIBUTE_WEAK
-extern void InitialMallocHook_Sbrk(void* result, ptrdiff_t increment);
+extern void InitialMallocHook_Sbrk(const void* result, ptrdiff_t increment);
+
+namespace base { namespace internal {
+template<typename PtrT>
+PtrT AtomicPtr<PtrT>::Exchange(PtrT new_val) {
+  base::subtle::MemoryBarrier();  // Release semantics.
+  // Depending on the system, NoBarrier_AtomicExchange(AtomicWord*)
+  // may have been defined to return an AtomicWord, Atomic32, or
+  // Atomic64.  We hide that implementation detail here with an
+  // explicit cast.  This prevents MSVC 2005, at least, from complaining.
+  PtrT old_val = reinterpret_cast<PtrT>(static_cast<AtomicWord>(
+      base::subtle::NoBarrier_AtomicExchange(
+          &data_,
+          reinterpret_cast<AtomicWord>(new_val))));
+  base::subtle::MemoryBarrier();  // And acquire semantics.
+  return old_val;
+}
+
+AtomicPtr<MallocHook::NewHook>    new_hook_ = {
+  reinterpret_cast<AtomicWord>(InitialMallocHook_New) };
+AtomicPtr<MallocHook::DeleteHook> delete_hook_ = { 0 };
+AtomicPtr<MallocHook::MmapHook>   mmap_hook_ = {
+  reinterpret_cast<AtomicWord>(InitialMallocHook_MMap) };
+AtomicPtr<MallocHook::MunmapHook> munmap_hook_ = { 0 };
+AtomicPtr<MallocHook::MremapHook> mremap_hook_ = { 0 };
+AtomicPtr<MallocHook::SbrkHook>   sbrk_hook_ = {
+  reinterpret_cast<AtomicWord>(InitialMallocHook_Sbrk) };
+
+} }  // namespace base::internal
+
+using base::internal::new_hook_;
+using base::internal::delete_hook_;
+using base::internal::mmap_hook_;
+using base::internal::munmap_hook_;
+using base::internal::mremap_hook_;
+using base::internal::sbrk_hook_;
+
+
+// static
+MallocHook::NewHook MallocHook::SetNewHook(NewHook hook) {
+  return new_hook_.Exchange(hook);
+}
+
+// static
+MallocHook::DeleteHook MallocHook::SetDeleteHook(DeleteHook hook) {
+  return delete_hook_.Exchange(hook);
+}
+
+// static
+MallocHook::MmapHook MallocHook::SetMmapHook(MmapHook hook) {
+  return mmap_hook_.Exchange(hook);
+}
+
+// static
+MallocHook::MunmapHook MallocHook::SetMunmapHook(MunmapHook hook) {
+  return munmap_hook_.Exchange(hook);
+}
+
+// static
+MallocHook::MremapHook MallocHook::SetMremapHook(MremapHook hook) {
+  return mremap_hook_.Exchange(hook);
+}
+
+// static
+MallocHook::SbrkHook MallocHook::SetSbrkHook(SbrkHook hook) {
+  return sbrk_hook_.Exchange(hook);
+}
 
-MallocHook::NewHook    MallocHook::new_hook_ = InitialMallocHook_New;
-MallocHook::DeleteHook MallocHook::delete_hook_ = NULL;
-MallocHook::MmapHook   MallocHook::mmap_hook_ = InitialMallocHook_MMap;
-MallocHook::MunmapHook MallocHook::munmap_hook_ = NULL;
-MallocHook::MremapHook MallocHook::mremap_hook_ = NULL;
-MallocHook::SbrkHook   MallocHook::sbrk_hook_ = InitialMallocHook_Sbrk;
 
 // The definitions of weak default malloc hooks (New, MMap, and Sbrk)
 // that self deinstall on their first call.  This is entirely for
@@ -109,13 +176,13 @@ MallocHook::SbrkHook   MallocHook::sbrk_hook_ = InitialMallocHook_Sbrk;
 // hook will be called, and do an if check, for every new.  Alas.
 // TODO(csilvers): add support for removing a hook from the middle of a chain.
 
-void InitialMallocHook_New(void* ptr, size_t size) {
+void InitialMallocHook_New(const void* ptr, size_t size) {
    if (MallocHook::GetNewHook() == &InitialMallocHook_New)
      MallocHook::SetNewHook(NULL);
 }
 
-void InitialMallocHook_MMap(void* result,
-                            void* start,
+void InitialMallocHook_MMap(const void* result,
+                            const void* start,
                             size_t size,
                             int protection,
                             int flags,
@@ -125,14 +192,16 @@ void InitialMallocHook_MMap(void* result,
     MallocHook::SetMmapHook(NULL);
 }
 
-void InitialMallocHook_Sbrk(void* result, ptrdiff_t increment) {
+void InitialMallocHook_Sbrk(const void* result, ptrdiff_t increment) {
   if (MallocHook::GetSbrkHook() == &InitialMallocHook_Sbrk)
     MallocHook::SetSbrkHook(NULL);
 }
 
-DECLARE_ATTRIBUTE_SECTION(google_malloc_allocators);
+DEFINE_ATTRIBUTE_SECTION_VARS(google_malloc);
+DECLARE_ATTRIBUTE_SECTION_VARS(google_malloc);
   // actual functions are in debugallocation.cc or tcmalloc.cc
-DECLARE_ATTRIBUTE_SECTION(malloc_hook_callers);
+DEFINE_ATTRIBUTE_SECTION_VARS(malloc_hook);
+DECLARE_ATTRIBUTE_SECTION_VARS(malloc_hook);
   // actual functions are in this file, malloc_hook.cc, and low_level_alloc.cc
 
 #define ADDR_IN_ATTRIBUTE_SECTION(addr, name) \
@@ -144,9 +213,9 @@ DECLARE_ATTRIBUTE_SECTION(malloc_hook_callers);
 // Return true iff 'caller' is a return address within a function
 // that calls one of our hooks via MallocHook:Invoke*.
 // A helper for GetCallerStackTrace.
-static inline bool InHookCaller(void* caller) {
-  return ADDR_IN_ATTRIBUTE_SECTION(caller, google_malloc_allocators) ||
-         ADDR_IN_ATTRIBUTE_SECTION(caller, malloc_hook_callers);
+static inline bool InHookCaller(const void* caller) {
+  return ADDR_IN_ATTRIBUTE_SECTION(caller, google_malloc) ||
+         ADDR_IN_ATTRIBUTE_SECTION(caller, malloc_hook);
   // We can use one section for everything except tcmalloc_or_debug
   // due to its special linkage mode, which prevents merging of the sections.
 }
@@ -157,14 +226,16 @@ static bool checked_sections = false;
 
 static inline void CheckInHookCaller() {
   if (!checked_sections) {
-    if (ATTRIBUTE_SECTION_START(google_malloc_allocators) ==
-        ATTRIBUTE_SECTION_STOP(google_malloc_allocators)) {
-      RAW_LOG(ERROR, "google_malloc_allocators section is missing, "
+    INIT_ATTRIBUTE_SECTION_VARS(google_malloc);
+    if (ATTRIBUTE_SECTION_START(google_malloc) ==
+        ATTRIBUTE_SECTION_STOP(google_malloc)) {
+      RAW_LOG(ERROR, "google_malloc section is missing, "
                      "thus InHookCaller is broken!");
     }
-    if (ATTRIBUTE_SECTION_START(malloc_hook_callers) == 
-        ATTRIBUTE_SECTION_STOP(malloc_hook_callers)) {
-      RAW_LOG(ERROR, "malloc_hook_callers section is missing, "
+    INIT_ATTRIBUTE_SECTION_VARS(malloc_hook);
+    if (ATTRIBUTE_SECTION_START(malloc_hook) ==
+        ATTRIBUTE_SECTION_STOP(malloc_hook)) {
+      RAW_LOG(ERROR, "malloc_hook section is missing, "
                      "thus InHookCaller is broken!");
     }
     checked_sections = true;
@@ -176,16 +247,17 @@ static inline void CheckInHookCaller() {
 // into the implementations for GetStackTrace instead of the skip_count.
 int MallocHook::GetCallerStackTrace(void** result, int max_depth,
                                     int skip_count) {
-#ifndef HAVE___ATTRIBUTE__
-    // Fall back to GetStackTrace and good old but fragile frame skip counts.
-    // Note: this path is inaccurate when a hook is not called directly by an
-    // allocation function but is daisy-chained through another hook,
-    // search for MallocHook::(Get|Set|Invoke)* to find such cases.
-    return GetStackTrace(result, max_depth, skip_count + int(DEBUG_MODE));
-             // due to -foptimize-sibling-calls in opt mode
-             // there's no need for extra frame skip here then
-  }
-#endif
+#if defined(NO_TCMALLOC_SAMPLES)
+  return 0;
+#elif !defined(HAVE_ATTRIBUTE_SECTION_START)
+  // Fall back to GetStackTrace and good old but fragile frame skip counts.
+  // Note: this path is inaccurate when a hook is not called directly by an
+  // allocation function but is daisy-chained through another hook,
+  // search for MallocHook::(Get|Set|Invoke)* to find such cases.
+  return GetStackTrace(result, max_depth, skip_count + int(DEBUG_MODE));
+  // due to -foptimize-sibling-calls in opt mode
+  // there's no need for extra frame skip here then
+#else
   CheckInHookCaller();
   // MallocHook caller determination via InHookCaller works, use it:
   static const int kMaxSkip = 32 + 6 + 3;
@@ -206,7 +278,7 @@ int MallocHook::GetCallerStackTrace(void** result, int max_depth,
       i += 1;  // skip hook caller frame
       depth -= i;  // correct depth
       if (depth > max_depth) depth = max_depth;
-      memcpy(result, stack+i, depth * sizeof(stack[0]));
+      copy(stack + i, stack + i + depth, result);
       if (depth < max_depth  &&  depth + i == kStackSize) {
         // get frames for the missing depth
         depth +=
@@ -216,8 +288,12 @@ int MallocHook::GetCallerStackTrace(void** result, int max_depth,
     }
   }
   RAW_LOG(WARNING, "Hooked allocator frame not found, returning empty trace");
-    // Try increasing kMaxSkip or else something must be wrong with InHookCaller
+    // If this happens try increasing kMaxSkip
+    // or else something must be wrong with InHookCaller,
+    // e.g. for every section used in InHookCaller
+    // all functions in that section must be inside the same library.
   return 0;
+#endif
 }
 
 // On Linux/x86, we override mmap/munmap/mremap/sbrk
@@ -229,7 +305,8 @@ int MallocHook::GetCallerStackTrace(void** result, int max_depth,
 // just call through to them.
 
 
-#if defined(__linux) && (defined(__i386__) || defined(__x86_64__))
+#if defined(__linux) && \
+    (defined(__i386__) || defined(__x86_64__) || defined(__PPC__))
 #include <unistd.h>
 #include <syscall.h>
 #include <sys/mman.h>
@@ -239,10 +316,10 @@ int MallocHook::GetCallerStackTrace(void** result, int max_depth,
 // The x86-32 case and the x86-64 case differ:
 // 32b has a mmap2() syscall, 64b does not.
 // 64b and 32b have different calling conventions for mmap().
-#if defined(__i386__)
+#if defined(__i386__) || defined(__PPC__)
 
 static inline void* do_mmap64(void *start, size_t length,
-                              int prot, int flags, 
+                              int prot, int flags,
                               int fd, __off64_t offset) __THROW {
   void *result;
 
@@ -300,23 +377,23 @@ static inline void* do_mmap64(void *start, size_t length,
 // stack are at the same offsets for all the calls of memory allocating
 // functions.
 
-// Put all callers of MallocHook::Invoke* in this module into 
-// malloc_hook_callers section,
+// Put all callers of MallocHook::Invoke* in this module into
+// malloc_hook section,
 // so that MallocHook::GetCallerStackTrace can function accurately:
 extern "C" {
   void* mmap64(void *start, size_t length, int prot, int flags,
                int fd, __off64_t offset  ) __THROW
-    ATTRIBUTE_SECTION(malloc_hook_callers);
+    ATTRIBUTE_SECTION(malloc_hook);
   void* mmap(void *start, size_t length,int prot, int flags,
              int fd, off_t offset) __THROW
-    ATTRIBUTE_SECTION(malloc_hook_callers);
+    ATTRIBUTE_SECTION(malloc_hook);
   int munmap(void* start, size_t length) __THROW
-    ATTRIBUTE_SECTION(malloc_hook_callers);
+    ATTRIBUTE_SECTION(malloc_hook);
   void* mremap(void* old_addr, size_t old_size, size_t new_size,
                int flags, ...) __THROW
-    ATTRIBUTE_SECTION(malloc_hook_callers);
+    ATTRIBUTE_SECTION(malloc_hook);
   void* sbrk(ptrdiff_t increment) __THROW
-    ATTRIBUTE_SECTION(malloc_hook_callers);
+    ATTRIBUTE_SECTION(malloc_hook);
 }
 
 extern "C" void* mmap64(void *start, size_t length, int prot, int flags,
