@@ -57,7 +57,7 @@
 // is just one more chance for the liveness flood to be inexact
 // (see the comment in our .h file).
 
-#include "config.h"
+#include "config_for_unittests.h"
 #include <sys/poll.h>
 #if defined HAVE_STDINT_H
 #include <stdint.h>             // to get uint16_t (ISO naming madness)
@@ -67,10 +67,17 @@
 #include <sys/types.h>
 #include <stdlib.h>
 #include <errno.h>              // errno
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>             // for sleep(), geteuid()
-#include <fcntl.h>              // for open(), close()
-#include <malloc.h>
+#endif
+#ifdef HAVE_MMAP
 #include <sys/mman.h>
+#endif
+#include <fcntl.h>              // for open(), close()
+// FreeBSD has malloc.h, but complains if you use it
+#if defined(HAVE_MALLOC_H) && !defined(__FreeBSD__)
+#include <malloc.h>
+#endif
 
 #include <netinet/in.h>         // inet_ntoa
 #include <arpa/inet.h>          // inet_ntoa
@@ -88,6 +95,8 @@
 #include <iomanip>              // for hex
 #include <set>
 #include <map>
+#include <list>
+#include <memory>
 #include <vector>
 #include <string>
 
@@ -97,9 +106,17 @@
 #include <google/heap-checker.h>
 #include "memory_region_map.h"
 #include <google/malloc_extension.h>
+#include <google/stacktrace.h>
+
+// On systems (like freebsd) that don't define MAP_ANONYMOUS, use the old
+// form of the name instead.
+#ifndef MAP_ANONYMOUS
+# define MAP_ANONYMOUS MAP_ANON
+#endif
 
 using namespace std;
 
+// ========================================================================= //
 
 // TODO(maxim): write a shell script to test that these indeed crash us
 //              (i.e. we do detect leaks)
@@ -141,6 +158,9 @@ DEFINE_bool(no_threads,
             // This is used so we can make can_create_leaks_reliably true
             // for any pthread implementation and test with that.
 
+DECLARE_int64(heap_check_max_pointer_offset);   // heap-checker.cc
+DECLARE_string(heap_check);  // in heap-checker.cc
+
 #define WARN_IF(cond, msg)   LOG_IF(WARNING, cond, msg)
 
 // This is an evil macro!  Be very careful using it...
@@ -149,6 +169,8 @@ DEFINE_bool(no_threads,
 // This is, likewise, evil
 #define LOGF         VLOG(INFO)
 
+static void RunHeapBusyThreads();  // below
+
 
 class Closure {
  public:
@@ -218,7 +240,7 @@ static bool can_create_leaks_reliably = false;
 // We use a simple allocation wrapper
 // to make sure we wipe out the newly allocated objects
 // in case they still happened to contain some pointer data
-// accidently left by the memory allocator.
+// accidentally left by the memory allocator.
 struct Initialized { };
 static Initialized initialized;
 void* operator new(size_t size, const Initialized&) {
@@ -276,22 +298,16 @@ static const uintptr_t kHideMask = 0xF03A5F7B;
 //  can be allocated at the same heap address.)
 template <class T>
 static void Hide(T** ptr) {
-  // We have to do memcpy rather than just casting here, because a cast
-  // is illegal (and can cause aliasing issues).
-  uintptr_t hidden = reinterpret_cast<uintptr_t>(*ptr) ^ kHideMask;
-  assert(sizeof(*ptr) == sizeof(hidden));
-  memcpy(ptr, &hidden, sizeof(*ptr));
+  // we cast values, not dereferenced pointers, so no aliasing issues:
+  *ptr = reinterpret_cast<T*>(reinterpret_cast<uintptr_t>(*ptr) ^ kHideMask);
   VLOG(2) << "hid: " << static_cast<void*>(*ptr);
 }
 
 template <class T>
 static void UnHide(T** ptr) {
   VLOG(2) << "unhiding: " << static_cast<void*>(*ptr);
-  // We have to do memcpy rather than just casting here, because a cast
-  // is illegal (and can cause aliasing issues).
-  uintptr_t unhidden = reinterpret_cast<uintptr_t>(*ptr) ^ kHideMask;
-  assert(sizeof(*ptr) == sizeof(unhidden));
-  memcpy(ptr, &unhidden, sizeof(*ptr));
+  // we cast values, not dereferenced pointers, so no aliasing issues:
+  *ptr = reinterpret_cast<T*>(reinterpret_cast<uintptr_t>(*ptr) ^ kHideMask);
 }
 
 static void LogHidden(const char* message, const void* ptr) {
@@ -357,7 +373,7 @@ static void DoDeAllocHidden(void** ptr) {
   void* p = *ptr;
   VLOG(2) << "Deallocating hidden " << p;
   UnHide(&p);
-  delete [] (char*)p;
+  delete [] reinterpret_cast<char*>(p);
 }
 
 static void DeAllocHidden(void** ptr) {
@@ -419,8 +435,8 @@ static void TestHeapLeakCheckerDeathSimple() {
 static void MakeDeathLoop(void** arr1, void** arr2) {
   void** a1 = new(initialized) void*[2];
   void** a2 = new(initialized) void*[2];
-  a1[1] = (void*)a2;
-  a2[1] = (void*)a1;
+  a1[1] = reinterpret_cast<void*>(a2);
+  a2[1] = reinterpret_cast<void*>(a1);
   Hide(&a1);
   Hide(&a2);
   Use(&a1);
@@ -458,7 +474,7 @@ static void TestHeapLeakCheckerDeathInverse() {
   LogHidden("Leaking", foo);
   DeAllocHidden(&bar);
   Pause();
-  VerifyLeaks(&check, SAME_HEAP, -150 * int64(sizeof(int)), 0);
+  VerifyLeaks(&check, SAME_HEAP, -150 * static_cast<int64>(sizeof(int)), 0);
   DeAllocHidden(&foo);
 }
 
@@ -656,10 +672,10 @@ static void ThreadDisabledLeaks() {
   if (FLAGS_no_threads)  return;
   pthread_t tid;
   pthread_attr_t attr;
-  CHECK(pthread_attr_init(&attr) == 0);
-  CHECK(pthread_create(&tid, &attr, RunDisabledLeaks, NULL) == 0);
+  CHECK_EQ(pthread_attr_init(&attr), 0);
+  CHECK_EQ(pthread_create(&tid, &attr, RunDisabledLeaks, NULL), 0);
   void* res;
-  CHECK(pthread_join(tid, &res) == 0);
+  CHECK_EQ(pthread_join(tid, &res), 0);
 }
 
 // different disabled leaks (some in threads)
@@ -748,9 +764,9 @@ static void TestSTLAllocInverse() {
 template<class Alloc>
 static void DirectTestSTLAlloc(Alloc allocator, const char* name) {
   HeapLeakChecker check((string("direct_stl-") + name).c_str());
-  const int size = 1000;
-  typename Alloc::pointer ptrs[size];
-  for (int i = 0; i < size; ++i) {
+  static const int kSize = 1000;
+  typename Alloc::pointer ptrs[kSize];
+  for (int i = 0; i < kSize; ++i) {
     typename Alloc::pointer p = allocator.allocate(i*3+1);
     HeapLeakChecker::IgnoreObject(p);
     // This will crash if p is not known to heap profiler:
@@ -758,7 +774,7 @@ static void DirectTestSTLAlloc(Alloc allocator, const char* name) {
     HeapLeakChecker::UnIgnoreObject(p);
     ptrs[i] = p;
   }
-  for (int i = 0; i < size; ++i) {
+  for (int i = 0; i < kSize; ++i) {
     allocator.deallocate(ptrs[i], i*3+1);
     ptrs[i] = NULL;
   }
@@ -766,24 +782,26 @@ static void DirectTestSTLAlloc(Alloc allocator, const char* name) {
 }
 
 static struct group* grp = NULL;
-static pthread_once_t key_once = PTHREAD_ONCE_INIT;
 static const int kKeys = 50;
 static pthread_key_t key[kKeys];
 
 static void KeyFree(void* ptr) {
-  delete [] (char*)ptr;
+  delete [] reinterpret_cast<char*>(ptr);
 }
 
+static bool key_init_has_run = false;
+
 static void KeyInit() {
   for (int i = 0; i < kKeys; ++i) {
     CHECK_EQ(pthread_key_create(&key[i], KeyFree), 0);
     VLOG(2) << "pthread key " << i << " : " << key[i];
   }
+  key_init_has_run = true;   // needed for a sanity-check
 }
 
 // force various C library static and thread-specific allocations
 static void TestLibCAllocate() {
-  pthread_once(&key_once, KeyInit);
+  CHECK(key_init_has_run);
   for (int i = 0; i < kKeys; ++i) {
     void* p = pthread_getspecific(key[i]);
     if (NULL == p) {
@@ -805,8 +823,8 @@ static void TestLibCAllocate() {
   inet_ntoa(addr);
   const time_t now = time(NULL);
   ctime(&now);
-  void *stack[1];
 #ifdef HAVE_EXECINFO_H
+  void *stack[1];
   backtrace(stack, 1);
 #endif
 #ifdef HAVE_GRP_H
@@ -822,6 +840,8 @@ static void TestLibCAllocate() {
 
 // Continuous random heap memory activity to try to disrupt heap checking.
 static void* HeapBusyThreadBody(void* a) {
+  const int thread_num = reinterpret_cast<intptr_t>(a);
+  VLOG(0) << "A new HeapBusyThread " << thread_num;
   TestLibCAllocate();
 
   int user = 0;
@@ -867,7 +887,7 @@ static void* HeapBusyThreadBody(void* a) {
             << reinterpret_cast<void*>(
                  reinterpret_cast<uintptr_t>(ptr) ^ kHideMask)
             << "^" << reinterpret_cast<void*>(kHideMask);
-    if (FLAGS_test_register_leak) {
+    if (FLAGS_test_register_leak  &&  thread_num % 5 == 0) {
       // Hide the register "ptr" value with an xor mask.
       // If one provides --test_register_leak flag, the test should
       // (with very high probability) crash on some leak check
@@ -902,17 +922,19 @@ static void* HeapBusyThreadBody(void* a) {
 }
 
 static void RunHeapBusyThreads() {
-  if (FLAGS_no_threads)  return;
+  KeyInit();
+  if (!FLAGS_interfering_threads || FLAGS_no_threads)  return;
 
   const int n = 17;  // make many threads
 
   pthread_t tid;
   pthread_attr_t attr;
-  CHECK(pthread_attr_init(&attr) == 0);
+  CHECK_EQ(pthread_attr_init(&attr), 0);
   // make them and let them run
   for (int i = 0; i < n; ++i) {
     VLOG(0) << "Creating extra thread " << i + 1;
-    CHECK(pthread_create(&tid, &attr, HeapBusyThreadBody, NULL) == 0);
+    CHECK(pthread_create(&tid, &attr, HeapBusyThreadBody,
+                         reinterpret_cast<void*>(i)) == 0);
   }
 
   Pause();
@@ -930,7 +952,7 @@ REGISTER_MODULE_INITIALIZER(heap_checker_unittest, {
 // and NamedThreeDisabledLeaks for the name-based disabling to work in opt mode
 // we need to undo the tail-recursion optimization effect
 // of -foptimize-sibling-calls that is enabled by -O2 in gcc 3.* and 4.*
-// so that for the leaking calls we can find the stack frame 
+// so that for the leaking calls we can find the stack frame
 // that resolves to a Named*DisabledLeaks function.
 // We do this by adding a fake last statement to these functions
 // so that tail-recursion optimization is done with it.
@@ -996,10 +1018,10 @@ static void ThreadNamedDisabledLeaks() {
   if (FLAGS_no_threads)  return;
   pthread_t tid;
   pthread_attr_t attr;
-  CHECK(pthread_attr_init(&attr) == 0);
-  CHECK(pthread_create(&tid, &attr, RunNamedDisabledLeaks, NULL) == 0);
+  CHECK_EQ(pthread_attr_init(&attr), 0);
+  CHECK_EQ(pthread_create(&tid, &attr, RunNamedDisabledLeaks, NULL), 0);
   void* res;
-  CHECK(pthread_join(tid, &res) == 0);
+  CHECK_EQ(pthread_join(tid, &res), 0);
 }
 
 // test leak disabling via function names
@@ -1027,10 +1049,62 @@ static void TestHeapLeakCheckerNamedDisabling() {
   }
 }
 
-// The code from here to main()
-// is to test that objects that are reachable from global
-// variables are not reported as leaks,
-// with the few exceptions like multiple-inherited objects.
+// ========================================================================= //
+
+// This code section is to test that objects that are reachable from global
+// variables are not reported as leaks
+// as well as that (Un)IgnoreObject work for such objects fine.
+
+// An object making functions:
+// returns a "weird" pointer to a new object for which
+// it's worth checking that the object is reachable via that pointer.
+typedef void* (*ObjMakerFunc)();
+static list<ObjMakerFunc> obj_makers;  // list of registered object makers
+
+// Helper macro to register an object making function
+// 'name' is an identifier of this object maker,
+// 'body' is its function body that must declare
+//        pointer 'p' to the nex object to return.
+// Usage example:
+//   REGISTER_OBJ_MAKER(trivial, int* p = new(initialized) int;)
+#define REGISTER_OBJ_MAKER(name, body) \
+  void* ObjMaker_##name##_() { \
+    VLOG(1) << "Obj making " << #name; \
+    body; \
+    return p; \
+  } \
+  static ObjMakerRegistrar maker_reg_##name##__(&ObjMaker_##name##_);
+// helper class for REGISTER_OBJ_MAKER
+struct ObjMakerRegistrar {
+  ObjMakerRegistrar(ObjMakerFunc obj_maker) { obj_makers.push_back(obj_maker); }
+};
+
+// List of the objects/pointers made with all the obj_makers
+// to test reachability via global data pointers during leak checks.
+static list<void*>* live_objects = new list<void*>;
+  // pointer so that it does not get destructed on exit
+
+// Exerciser for one ObjMakerFunc.
+static void TestPointerReach(ObjMakerFunc obj_maker) {
+  HeapLeakChecker::IgnoreObject(obj_maker());  // test IgnoreObject
+
+  void* obj = obj_maker();
+  HeapLeakChecker::IgnoreObject(obj);
+  HeapLeakChecker::UnIgnoreObject(obj);  // test UnIgnoreObject
+  HeapLeakChecker::IgnoreObject(obj);  // not to need deletion for obj
+
+  live_objects->push_back(obj_maker());  // test reachability at leak check
+}
+
+// Test all ObjMakerFunc registred via REGISTER_OBJ_MAKER.
+static void TestObjMakers() {
+  for (list<ObjMakerFunc>::const_iterator i = obj_makers.begin();
+       i != obj_makers.end(); ++i) {
+    TestPointerReach(*i);
+    TestPointerReach(*i);  // a couple more times would not hurt
+    TestPointerReach(*i);
+  }
+}
 
 // A dummy class to mimic allocation behavior of string-s.
 template<class T>
@@ -1072,37 +1146,59 @@ struct Array {
   T* ptr;
 };
 
-static Array<char>* live_leak = NULL;
-static Array<char>* live_leak2 = new(initialized) Array<char>();
-static int* live_leak3 = new(initialized) int[10];
-static const char* live_leak4 = new(initialized) char[5];
-static int data[] = { 1, 2, 3, 4, 5, 6, 7, 21, 22, 23, 24, 25, 26, 27 };
-static set<int> live_leak5(data, data+7);
-static const set<int> live_leak6(data, data+14);
-static const Array<char>* live_leak_arr1 = new(initialized) Array<char>[5];
+// to test pointers to objects, built-in arrays, string, etc:
+REGISTER_OBJ_MAKER(plain, int* p = new(initialized) int;)
+REGISTER_OBJ_MAKER(int_array_1, int* p = new(initialized) int[1];)
+REGISTER_OBJ_MAKER(int_array, int* p = new(initialized) int[10];)
+REGISTER_OBJ_MAKER(string, Array<char>* p = new(initialized) Array<char>();)
+REGISTER_OBJ_MAKER(string_array,
+                   Array<char>* p = new(initialized) Array<char>[5];)
+REGISTER_OBJ_MAKER(char_array, char* p = new(initialized) char[5];)
+REGISTER_OBJ_MAKER(appended_string,
+  Array<char>* p = new Array<char>();
+  p->append(Array<char>());
+)
+REGISTER_OBJ_MAKER(plain_ptr, int** p = new(initialized) int*;)
+REGISTER_OBJ_MAKER(linking_ptr,
+  int** p = new(initialized) int*;
+  *p = new(initialized) int;
+)
+
+// small objects:
+REGISTER_OBJ_MAKER(0_sized, void* p = malloc(0);)  // 0-sized object (important)
+REGISTER_OBJ_MAKER(1_sized, void* p = malloc(1);)
+REGISTER_OBJ_MAKER(2_sized, void* p = malloc(2);)
+REGISTER_OBJ_MAKER(3_sized, void* p = malloc(3);)
+REGISTER_OBJ_MAKER(4_sized, void* p = malloc(4);)
+
+static int set_data[] = { 1, 2, 3, 4, 5, 6, 7, 21, 22, 23, 24, 25, 26, 27 };
+static set<int> live_leak_set(set_data, set_data+7);
+static const set<int> live_leak_const_set(set_data, set_data+14);
+
+REGISTER_OBJ_MAKER(set,
+  set<int>* p = new(initialized) set<int>(set_data, set_data + 13);
+)
 
 class ClassA {
  public:
-  ClassA(int a) : ptr(NULL) { }
+  explicit ClassA(int a) : ptr(NULL) { }
   mutable char* ptr;
 };
-
-static const ClassA live_leak7(1);
+static const ClassA live_leak_mutable(1);
 
 template<class C>
 class TClass {
  public:
-  TClass(int a) : ptr(NULL) { }
+  explicit TClass(int a) : ptr(NULL) { }
   mutable C val;
   mutable C* ptr;
 };
-
-static const TClass<Array<char> > live_leak8(1);
+static const TClass<Array<char> > live_leak_templ_mutable(1);
 
 class ClassB {
  public:
   ClassB() { }
-  int b[10];
+  char b[7];
   virtual void f() { }
   virtual ~ClassB() { }
 };
@@ -1110,116 +1206,175 @@ class ClassB {
 class ClassB2 {
  public:
   ClassB2() { }
-  int b2[10];
+  char b2[11];
   virtual void f2() { }
   virtual ~ClassB2() { }
 };
 
 class ClassD1 : public ClassB {
-  int d1[10];
+  char d1[15];
   virtual void f() { }
 };
 
 class ClassD2 : public ClassB2 {
-  int d2[10];
+  char d2[19];
   virtual void f2() { }
 };
 
 class ClassD : public ClassD1, public ClassD2 {
-  int d[10];
+  char d[3];
   virtual void f() { }
   virtual void f2() { }
 };
 
-static ClassB* live_leak_b;
-static ClassD1* live_leak_d1;
-static ClassD2* live_leak_d2;
-static ClassD* live_leak_d;
+// to test pointers to objects of base subclasses:
 
-static ClassB* live_leak_b_d1;
-static ClassB2* live_leak_b2_d2;
-static ClassB* live_leak_b_d;
-static ClassB2* live_leak_b2_d;
+REGISTER_OBJ_MAKER(B,  ClassB*  p = new(initialized) ClassB;)
+REGISTER_OBJ_MAKER(D1, ClassD1* p = new(initialized) ClassD1;)
+REGISTER_OBJ_MAKER(D2, ClassD2* p = new(initialized) ClassD2;)
+REGISTER_OBJ_MAKER(D,  ClassD*  p = new(initialized) ClassD;)
 
-static ClassD1* live_leak_d1_d;
-static ClassD2* live_leak_d2_d;
+REGISTER_OBJ_MAKER(D1_as_B,  ClassB*  p = new(initialized) ClassD1;)
+REGISTER_OBJ_MAKER(D2_as_B2, ClassB2* p = new(initialized) ClassD2;)
+REGISTER_OBJ_MAKER(D_as_B,   ClassB*  p = new(initialized)  ClassD;)
+REGISTER_OBJ_MAKER(D_as_D1,  ClassD1* p = new(initialized) ClassD;)
+// inside-object pointers:
+REGISTER_OBJ_MAKER(D_as_B2,  ClassB2* p = new(initialized) ClassD;)
+REGISTER_OBJ_MAKER(D_as_D2,  ClassD2* p = new(initialized) ClassD;)
 
-// have leaks but ignore the leaked objects
-static void IgnoredLeaks() {
-  int* p = new(initialized) int[1];
-  HeapLeakChecker::IgnoreObject(p);
-  int** leak = new(initialized) int*;
-  HeapLeakChecker::IgnoreObject(leak);
-  *leak = new(initialized) int;
-  HeapLeakChecker::UnIgnoreObject(p);
-  delete [] p;
-}
+class InterfaceA {
+ public:
+  virtual void A() = 0;
+  virtual ~InterfaceA() { }
+ protected:
+  InterfaceA() { }
+};
+
+class InterfaceB {
+ public:
+  virtual void B() = 0;
+  virtual ~InterfaceB() { }
+ protected:
+  InterfaceB() { }
+};
+
+class InterfaceC : public InterfaceA {
+ public:
+  virtual void C() = 0;
+  virtual ~InterfaceC() { }
+ protected:
+  InterfaceC() { }
+};
+
+class ClassMltD1 : public ClassB, public InterfaceB, public InterfaceC {
+ public:
+  char d1[11];
+  virtual void f() { }
+  virtual void A() { }
+  virtual void B() { }
+  virtual void C() { }
+};
+
+class ClassMltD2 : public InterfaceA, public InterfaceB, public ClassB {
+ public:
+  char d2[15];
+  virtual void f() { }
+  virtual void A() { }
+  virtual void B() { }
+};
+
+// to specifically test heap reachability under
+// inerface-only multiple inheritance (some use inside-object pointers):
+REGISTER_OBJ_MAKER(MltD1,       ClassMltD1* p = new(initialized) ClassMltD1;)
+REGISTER_OBJ_MAKER(MltD1_as_B,  ClassB*     p = new(initialized) ClassMltD1;)
+REGISTER_OBJ_MAKER(MltD1_as_IA, InterfaceA* p = new(initialized) ClassMltD1;)
+REGISTER_OBJ_MAKER(MltD1_as_IB, InterfaceB* p = new(initialized) ClassMltD1;)
+REGISTER_OBJ_MAKER(MltD1_as_IC, InterfaceC* p = new(initialized) ClassMltD1;)
+
+REGISTER_OBJ_MAKER(MltD2,       ClassMltD2* p = new(initialized) ClassMltD2;)
+REGISTER_OBJ_MAKER(MltD2_as_B,  ClassB*     p = new(initialized) ClassMltD2;)
+REGISTER_OBJ_MAKER(MltD2_as_IA, InterfaceA* p = new(initialized) ClassMltD2;)
+REGISTER_OBJ_MAKER(MltD2_as_IB, InterfaceB* p = new(initialized) ClassMltD2;)
+
+// to mimic UnicodeString defined in third_party/icu,
+// which store a platform-independent-sized refcount in the first
+// few bytes and keeps a pointer pointing behind the refcount.
+REGISTER_OBJ_MAKER(unicode_string,
+  char* p = new char[sizeof(uint32) * 10];
+  p += sizeof(uint32);
+)
+// similar, but for platform-dependent-sized refcount
+REGISTER_OBJ_MAKER(ref_counted,
+  char* p = new char[sizeof(int) * 20];
+  p += sizeof(int);
+)
+
+struct Nesting {
+  struct Inner {
+    Nesting* parent;
+    Inner(Nesting* p) : parent(p) {}
+  };
+  Inner i0;
+  char n1[5];
+  Inner i1;
+  char n2[11];
+  Inner i2;
+  char n3[27];
+  Inner i3;
+  Nesting() : i0(this), i1(this), i2(this), i3(this) {}
+};
+
+// to test inside-object pointers pointing at objects nested into heap objects:
+REGISTER_OBJ_MAKER(nesting_i0, Nesting::Inner* p = &((new Nesting())->i0);)
+REGISTER_OBJ_MAKER(nesting_i1, Nesting::Inner* p = &((new Nesting())->i1);)
+REGISTER_OBJ_MAKER(nesting_i2, Nesting::Inner* p = &((new Nesting())->i2);)
+REGISTER_OBJ_MAKER(nesting_i3, Nesting::Inner* p = &((new Nesting())->i3);)
 
 // allocate many objects reachable from global data
 static void TestHeapLeakCheckerLiveness() {
-  live_leak_b = new(initialized) ClassB;
-  live_leak_d1 = new(initialized) ClassD1;
-  live_leak_d2 = new(initialized) ClassD2;
-  live_leak_d = new(initialized) ClassD;
+  live_leak_mutable.ptr = new(initialized) char[77];
+  live_leak_templ_mutable.ptr = new(initialized) Array<char>();
+  live_leak_templ_mutable.val = Array<char>();
 
-  live_leak_b_d1 = new(initialized) ClassD1;
-  live_leak_b2_d2 = new(initialized) ClassD2;
-  live_leak_b_d = new(initialized) ClassD;
-  live_leak_b2_d = new(initialized) ClassD;
-
-  live_leak_d1_d = new(initialized) ClassD;
-  live_leak_d2_d = new(initialized) ClassD;
-
-  HeapLeakChecker::IgnoreObject((ClassD*)live_leak_b2_d);
-  HeapLeakChecker::IgnoreObject((ClassD*)live_leak_d2_d);
-    // These two do not get deleted with liveness flood
-    // because the base class pointer points inside of the objects
-    // in such cases of multiple inheritance.
-    // Luckily google code does not use multiple inheritance almost at all.
-
-  live_leak = new(initialized) Array<char>();
-  delete [] live_leak3;
-  live_leak3 = new(initialized) int[33];
-  live_leak2->append(*live_leak);
-  live_leak7.ptr = new(initialized) char[77];
-  live_leak8.ptr = new(initialized) Array<char>();
-  live_leak8.val = Array<char>();
-
-  IgnoredLeaks();
-  IgnoredLeaks();
-  IgnoredLeaks();
+  TestObjMakers();
 }
 
-DECLARE_string(heap_check);
+// ========================================================================= //
 
-static void* Mmapper() {
+// Get address (PC value) following the mmap call into addr_after_mmap_call
+static void* Mmapper(uintptr_t* addr_after_mmap_call) {
   void* r = mmap(NULL, 100, PROT_READ|PROT_WRITE,
                  MAP_PRIVATE|MAP_ANONYMOUS, -1, 0);
+  // Get current PC value into addr_after_mmap_call
+  void* stack[1];
+  CHECK_EQ(GetStackTrace(stack, 1, 0), 1);
+  *addr_after_mmap_call = reinterpret_cast<uintptr_t>(stack[0]);
   sleep(0);  // undo -foptimize-sibling-calls
   return r;
 }
 
 // to trick complier into preventing inlining
-static void* (*mmapper_addr)() = &Mmapper;
+static void* (*mmapper_addr)(uintptr_t* addr) = &Mmapper;
 
 // TODO(maxim): copy/move this to memory_region_map_unittest
 // TODO(maxim): expand this test to include mmap64, mremap and sbrk calls.
 static void VerifyMemoryRegionMapStackGet() {
-  void* addr = (*mmapper_addr)();
+  uintptr_t caller_addr_limit;
+  void* addr = (*mmapper_addr)(&caller_addr_limit);
   uintptr_t caller = 0;
-  MemoryRegionMap::Lock();
-  for (MemoryRegionMap::RegionIterator
-         i = MemoryRegionMap::BeginRegionLocked();
-         i != MemoryRegionMap::EndRegionLocked(); ++i) {
-    if (i->start_addr == reinterpret_cast<uintptr_t>(addr)) {
-      CHECK(caller == 0);
-      caller = i->caller;
+  { MemoryRegionMap::LockHolder l;
+    for (MemoryRegionMap::RegionIterator
+           i = MemoryRegionMap::BeginRegionLocked();
+           i != MemoryRegionMap::EndRegionLocked(); ++i) {
+      if (i->start_addr == reinterpret_cast<uintptr_t>(addr)) {
+        CHECK_EQ(caller, 0);
+        caller = i->caller();
+      }
     }
   }
-  MemoryRegionMap::Unlock();
   // caller must point into Mmapper function:
   if (!(reinterpret_cast<uintptr_t>(mmapper_addr) <= caller  &&
-        caller <= reinterpret_cast<uintptr_t>(mmapper_addr) + 0x40)) {
+        caller < caller_addr_limit)) {
     LOGF << std::hex << "0x" << caller
          << " does not seem to point into code of function Mmapper at "
          << "0x" << reinterpret_cast<uintptr_t>(mmapper_addr)
@@ -1229,25 +1384,30 @@ static void VerifyMemoryRegionMapStackGet() {
   munmap(addr, 100);
 }
 
-static void* Mallocer() {
+static void* Mallocer(uintptr_t* addr_after_malloc_call) {
   void* r = malloc(100);
   sleep(0);  // undo -foptimize-sibling-calls
+  // Get current PC value into addr_after_malloc_call
+  void* stack[1];
+  CHECK_EQ(GetStackTrace(stack, 1, 0), 1);
+  *addr_after_malloc_call = reinterpret_cast<uintptr_t>(stack[0]);
   return r;
 }
 
 // to trick complier into preventing inlining
-static void* (*mallocer_addr)() = &Mallocer;
+static void* (*mallocer_addr)(uintptr_t* addr) = &Mallocer;
 
 // non-static for friendship with HeapProfiler
 // TODO(maxim): expand this test to include
 // realloc, calloc, memalign, valloc, pvalloc, new, and new[].
 extern void VerifyHeapProfileTableStackGet() {
-  void* addr = (*mallocer_addr)();
+  uintptr_t caller_addr_limit;
+  void* addr = (*mallocer_addr)(&caller_addr_limit);
   uintptr_t caller =
     reinterpret_cast<uintptr_t>(HeapLeakChecker::GetAllocCaller(addr));
   // caller must point into Mallocer function:
   if (!(reinterpret_cast<uintptr_t>(mallocer_addr) <= caller  &&
-        caller <= reinterpret_cast<uintptr_t>(mallocer_addr) + 0x40)) {
+        caller < caller_addr_limit)) {
     LOGF << std::hex << "0x" << caller
          << " does not seem to point into code of function Mallocer at "
          << "0x" << reinterpret_cast<uintptr_t>(mallocer_addr)
@@ -1257,6 +1417,8 @@ extern void VerifyHeapProfileTableStackGet() {
   free(addr);
 }
 
+// ========================================================================= //
+
 // Helper to do 'return 0;' inside main(): insted we do 'return Pass();'
 static int Pass() {
   fprintf(stdout, "PASS\n");
@@ -1276,6 +1438,8 @@ int main(int argc, char** argv) {
     VerifyHeapProfileTableStackGet();
   }
 
+  KeyInit();
+
   // glibc 2.4, on x86_64 at least, has a lock-ordering bug, which
   // means deadlock is possible when one thread calls dl_open at the
   // same time another thread is calling dl_iterate_phdr.  libunwind
@@ -1323,6 +1487,13 @@ int main(int argc, char** argv) {
       // (when !FLAGS_test_cancel_global_check)
   }
 
+  if (FLAGS_test_register_leak) {
+    // make us fail only where the .sh test expects:
+    Pause();
+    for (int i = 0; i < 20; ++i) CHECK(HeapLeakChecker::NoGlobalLeaks());
+    return Pass();
+  }
+
   TestHeapLeakCheckerLiveness();
 
   HeapLeakChecker heap_check("all");
@@ -1364,7 +1535,8 @@ int main(int argc, char** argv) {
   // Test that various STL allocators work.  Some of these are redundant, but
   // we don't know how STL might change in the future.  For example,
   // http://wiki/Main/StringNeStdString.
-#define DTSL(a) { DirectTestSTLAlloc(a, #a); Pause(); }
+#define DTSL(a) { DirectTestSTLAlloc(a, #a); \
+                  Pause(); }
   DTSL(std::allocator<char>());
   DTSL(std::allocator<int>());
   DTSL(std::string().get_allocator());
@@ -1402,10 +1574,6 @@ int main(int argc, char** argv) {
   range_disable_named = true;
   ThreadNamedDisabledLeaks();
 
-  HeapLeakChecker::IgnoreObject(new(initialized) set<int>(data, data + 13));
-    // This checks both that IgnoreObject works, and
-    // and the fact that we don't drop such leaks as live for some reason.
-
   CHECK(HeapLeakChecker::NoGlobalLeaks());  // so far, so good
 
   return Pass();
