@@ -37,13 +37,16 @@
 // another testing script to actually verify correctness.  See, eg,
 // heap-profiler_unittest.sh.
 
+#include "config_for_unittests.h"
 #include <stdlib.h>
 #include <stdio.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>                 // for fork()
+#endif
 #include <sys/wait.h>               // for wait()
 #include <google/heap-profiler.h>
 
-const static int kMaxCount = 100000;
+static const int kMaxCount = 100000;
 int* g_array[kMaxCount];              // an array of int-vectors
 
 static void Allocate(int start, int end, int size) {
